package com.sifang.modules.customer.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.sifang.modules.customer.form.CustomerEntryComponentVo;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.validator.constraints.NotBlank;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Entity
@Table(name = "crm_customer_entry")
public class CustomerEntry implements Serializable {

    @Id
    @GeneratedValue(generator = "idGenerator")
    @GenericGenerator(name = "idGenerator", strategy = "uuid")
    private String id;

    /**
     * 项目id
     */
    private String projectId;

    /**
     * 产品名称
     */
    private String projectName;

    /**
     * 产品编号
     */
    private String projectCode;

    private  int projectNum;

    /**
     * 代理商id
     */
    private String distributorId;

    /**
     * 代理商名称
     */
    private String traderName;

    /**
     * 代理商电话
     */
    private String traderTel;

    /**
     * 电站编码
     */
    private String powerStationCode;

    /**
     * 租户id
     */
    private String tenantId;

    /**
     * 租户分类: 1.个人 2.单位
     */
    private String tenantType;

    /**
     * 租户名称
     */
    @NotBlank(message = "业主姓名不能为空")
    private String tenantName;

    /**
     * 省
     */
    private String province;

    /**
     * 市
     */
    private String city;

    /**
     * 区县
     */
    private String area;

    /**
     * 详细地址
     */
    private String address;

    private String street;

    /**
     * 经度
     */
    private BigDecimal longitude;

    /**
     * 纬度
     */
    private BigDecimal latitude;

    /**
     * 联系方式
     */
//    @NotBlank(message = "联系方式不能为空")
    private String mobile;

    /**
     * 身份证号
     */
//    @NotBlank(message = "身份证号不能为空")
//    @Pattern(regexp = "(^\\d{15}$)|(^\\d{18}$)|(^\\d{17}(\\d|X|x)$)", message = "身份证号格式不正确")
    private String idCode;

    /**
     * 备注
     */
    private String remarks;

    private String encryptionIdCard;
    /**
     * 身份证照片正面
     */
    private String idCardFront;

    /**
     * 身份证照片正反面
     */
    private String idCardBack;

    /**
     * 状态 0.暂存 1.待审核 2.审核不通过 3.待下单 4.待收货 5.待安装 6.待并网 7.待验收 8.待交付 9.待归档 10.验收驳回 11.归档驳回 a.已完成 b.已删除 c.待上传发票 d.待签署合同
     */
    private String status;


    private String toExamineStatus;

    /**
     * 进件申请状态  0.暂存 1.未申请 2.待进件审核 3.进件审核不通过  NW:未审批;AP:审批中;RJ:拒绝;QY:签约;PR:审批通过;LS-已完成放款;LF-放款失败;
     */
    private String applyStatus;

    /**
     * 是否提交准入 0.否 1.是
     */
    private String admitFlag;

    /**
     * 创建人userId
     */
    private String createBy;

    /**
     * 创建人姓名
     */
    private String creator;

    /**
     * 创建时间
     */
    private Date createTime;

    /**
     * 设计方案 1.标准方案  2.组合方案
     */
    private String schemeType;

    /**
     * 方案类型id
     */
    private String schemeTypeId;

    /**
     * 方案类型名称
     */
    private String schemeTypeName;

    /**
     * 方案id
     */
    private String schemeId;

    /**
     * 方案名称
     */
    private String schemeName;

    /**
     * 设计图纸
     */
    private String designDrawings;

    /**
     * 设计图纸名称
     */
    private String designDrawingsName;

    /**
     * 验收报告id
     */
    private String receivingReportId;

    /**
     * 提交审核时间
     */
    private Date toExamineTime;

    /**
     * 审核人
     */
    private String examine;

    /**
     * 审核人名称
     */
    private String examineName;

    /**
     * 审核时间
     */
    private Date examineTime;

    /**
     * 设计审核备注
     */
    private String examineRemarks;

    /**
     * 设计自审备注
     */
    private String auditRemarks;

    /**
     * 下单人
     */
    private Date orderTime;

    /**
     * 下单人user_id
     */
    private String orderBy;

    /**
     * 下单人姓名
     */
    private String orderName;

    /**
     * 收货人user_id
     */
    private String receiveBy;

    /**
     * 收货时间
     */
    private Date receiveTime;

    /**
     * 收货人姓名
     */
    private String receiveName;

    /**
     * 是否派工：0.否 1.是
     */
    private String dispatchStatus;


    /**
     * 派工时间
     */
    private Date dispatchTime;

    /**
     * 派工工程师姓名，多个逗号拼接
     */
    private String engineerName;

    /**
     * 派工工程师id，多个逗号拼接
     */
    private String engineerId;

    /**
     * 安装人
     */
    private String installBy;

    /**
     * 安装时间
     */
    private Date installTime;

    /**
     * 并网人user_id
     */
    private String mergeBy;

    /**
     * 并网人姓名
     */
    private String mergeName;

    /**
     * 并网时间
     */
    private Date mergeTime;

    /**
     * 验收人
     */
    private String checkAcceptBy;

    /**
     * 验收人姓名
     */
    private String checkAcceptName;

    /**
     * 验收时间
     */
    private Date checkAcceptTime;

    /**
     * 验收报告地址
     */
    private String checkAcceptImgs;

    /**
     * 交付人
     */
    private String deliverBy;

    /**
     * 交付人姓名
     */
    private String deliverName;

    /**
     * 交付时间
     */
    private Date deliverTime;

    /**
     * 交付文件
     */
    private String deliverFile;

    /**
     * 归档人
     */
    private String indocBy;

    /**
     * 归档人姓名
     */
    private String indocName;

    /**
     * 归档时间
     */
    private Date indocTime;

    /**
     * 标记类型id
     */
    private String flag;

    /**
     * 信息一致性 1.否 2.是
     */
    private String uniformity;

    /**
     * 组件规格
     */
    private String componentSpec;

    /**
     * 设计容量
     */
    private Integer designCapacity;

    /**
     * 实际容量
     */
    private BigDecimal actualCapacity;

    /**
     * 方案备注
     */
    private String schemeRemarks;

    /**
     * 发电户号
     */
    private String generatorNo;

    /**
     * 并网机构 0 国网 1 南网
     */

    private String gridInstitution;

    /**
     * 电厂编码
     */
    private String powerPlantCode;

    private String provinceCode;

    /**
     * 是否支付 1.否 2.是
     */
    private String whetherToPay;

    /**
     * 申请重审 1.否 2.是 3.已通过申请 4.未通过',
     */
    private String retrial;

    /**
     * 申请原因id
     */
    private String applyReasonId;

    /**
     * 申请原因内容
     */
    private String applyReasonName;

    /**
     * 申请时间
     */
    private Date applyTime;

    /**
     * 申请整改 1.否 2.是 3.已通过申请 4.未通过',
     */
    private String applyChange;

    /**
     * 变更类型id
     */
    private String rectificationSetId;

    /**
     * 变更原因
     */
    private String changeReason;

    /**
     * 审核意见
     */
    private String auditComments;

    /**
     * 申请整改时间
     */
    private Date changeTime;

    /**
     * 是否接收：0否 1是
     */
    private String acceptFlag;

    /**
     * 接收时间
     */
    private Date acceptTime;

    /**
     * 接收人user_id
     */
    private String acceptBy;

    /**
     * 接收人
     */
    private String acceptor;

    /**
     * 合作商id
     */
    private String partnerId;

    /**
     * 合作商名称
     */
    private String partnerName;

    /**
     * 资方类型 1.越秀 2.浙江浙银金融租赁股份有限公司 3.中信
     */
    private String managementType;

    /**
     * 预审状态 1.待预审 2.预审中 3.预审不通过 4.预审通过
     */
    private String credit;

    /**
     * 信息使用授权状态：1.未授权 2.已发起 3.已授权
     */
    private String authorization;

    /**
     * 征信结果备注原因（不通过原因）
     */
    private String creditDsc;

    /**
     * 第一次进件状态(NEW-待审批、APPROVING - 审批中、APPROVED - 审批通过、REJECTED - 审批拒绝、CLOSED - 取消、APPROVED_RETURN - 退回、CANCEL - 撤回)
     */
    private String firstApplyStatus;

    /**
     * 第二次进件状态(NEW-待审批、APPROVING-审批中、APPROVED-审批通过、REJECTED-审批拒绝、CANCEL - 撤回、WAITING-等待推送/补充并网及投放信息)
     */
    private String nextApplyStatus;

    /**
     * 进件退回、驳回原因
     */
    private String applyBackReason;


    private Date applyBackTime;

    /**
     * 合同
     */
    private String contract;
    private String contractFilename;

    /**
     * 发票
     */
    private String bill;

    /**
     * 发票编号
     */
    private String billNumber;

    //是否有自用并网箱 1无 2有
    private String oneselfFlag;


    private String regionId;

    private BigDecimal epcPrice;//EPC单价/ 每W定价

    private String projectCompanyId;//项目公司id

    private String projectCompanyName;//项目公司名称

    private String productLineId;//产品线id

    private String productLineName;//产品线名称

    private String productLine;//业务属性 1.纯租25年业务、2.经营性租赁、3.融资租赁、4.光伏贷、5.全款、6.其他

    private String payCycle;//租金支付周期 1按月支付 2按季支付 3按年支付


    private String signStatus; // 合同签署状态：0.未邀请 1.待农户签约 2.待公司签约 3.已完成

    private String paymentStatus;//回款状态 0.默认 页面显示-- 1.等待放款 2.请款完成 3.已放款

    private String sn;// sn码序列号(多个用逗号拼接)

    private String nbqSn;//逆变器sn码

    private BigDecimal etd;//当期新增发电量

    private BigDecimal eto;//累计发电量

    private BigDecimal etdc;//当期新增发电收入

    private BigDecimal etoc;//累计发电收入

    private String warehouseId;

    private String gdtRemarks;

    private String bankName;//开户银行
    private String bankCode;//银行账号

    private String gender;//性别:MALE:男 FEMALE:女 UNKNOW:未知
    private Integer age;//年龄

    private String residenceBooklet;//户口本复印件
    private String tssBankNum;//联行号

    private String tssBankNumName; //开户支行
    private String bankCard;//银行卡照片
    private String insuranceFlag;//保险购买情况:1.控制投放及续保 2.控制投放 3.其他 4.投放后补充及续保
    @JsonFormat(pattern="yyyy-MM-dd")
    private Date leaseDate;//预计投放日
    private String bpName;//共签人名称
    private String bpRelation;//共签人与业主关系
    private String bpIdCard;//共签人证件号码,不能与租户身份证号码相同
    private String bpMobile;//共签人手机号
    private String bpCardFront;//共签人身份证正面
    private String bpCardBack;//共签人身份证反面
    private String bpFile;//共签人子女证明材料

    private String secCardStatus;//开卡状态（0未开卡 1已开卡 3开卡中 4开卡失败）

    private String zjpbImg;//组件排布图
    private String zjzxImg;//组件接线图
    private String zjcsImg;//组件支架图
    private String zjhsImg;//一次接线图/电气图
    private String nbqwzImg;//逆变器位置
    private String pdxwzImg;//配电箱位置
    private String filingType;//备案类型:1.农户备案 2.项目公司备案
    private String addressConsistent;//购售电地址是否一致 Y:是、N:否

    private String electricProvince;//购售电地址(省）
    private String electricCity;//购售电地址(市）
    private String electricArea;//购售电地址(区）
    private String electricAddress;//购售电地址(详细地址）

    private String createTraderId;
    private String createTraderName;
    private String bwSettleFlag;//是否并网结算 0.未结算 1.已结算
    private String wgSettleFlag;//是否完工结算 0.未结算 1.已结算 针对华融多产品用

    /*
    设计审核状态：1.待踏勘 2.待自审 3.自审驳回 4.自审通过 5.设计审核驳回 6.档案审核驳回 7.已完成
     */
    private String designAuditFlag;
    /**
     * 安装状态：0.未开始 1.待派工 2.待安装 3.自审驳回 4.安装完成
     */
    private String installFlag;
    /**
     * 安装验收状态：0.未开始 1.待自审 2.自审通过 3.验收驳回 4.验收通过
     */
    private String checkAcceptFlag;
    /**
     * 并网验收状态：0.未开始 1.待并网验收 2.并网验收驳回 3.资方审核中 4.资方审核通过
     */
    private String mergeFlag;

    private String onGridFlag;

    /**
     * 法人/组织负责人姓名，当前承租人为法人时要求必填
     */
    private String corp;

    /**
     * 法人登记注册号码
     */
    private String regno;

    private String roofType;

    /**
     * 物料安装状态:0.未完成 1.已完成
     */
    private String materialFixFlag;

    private String zsResult; // 自审结果：0.无 1.通过 2.驳回至勘测 3.驳回至设计

    /**
     * 物料并网状态:0.未完成 1.已完成
     */
    private String materialMergeFlag;

    // 上传4-6张，要包含：组件排布图、组件走线图、支架侧视图、支架后视图
    private String dzImgs;

    public String getOnGridFlag() {
        return onGridFlag;
    }

    public void setOnGridFlag(String onGridFlag) {
        this.onGridFlag = onGridFlag;
    }

    @Transient
    @OneToMany(targetEntity = ComposePlan.class)
    private List<ComposePlan> composePlanLists;

    @Transient
    private String type;//0:暂存 1:提交 3:初勘

    private String nbqDzId; // 接入逆变器平台之后返回的电站id

    private String nbqFactory; //逆变器所属厂家：00.无 01.锦浪云

    private Date preAuditTime; //预审通过时间

    private Date materialFixFinishTime; //物料安装完成时间

    private String ifRewardSettle; //是否已经奖励结算 0否 1是

    private Date firstGenerationTime;//首日发电时间

    private Date inComingTime;//进件时间

    private Date installAuditFinishTime;//安装提交验收时间(安装自审通过 最新一次操作时间)

    private Date installAcceptTime;//安装验收时间(安装验收通过的时间)

    private String exigenceName;//紧急联系人

    private String exigencePhone;//紧急联系人电话

    /**
     * 出生日期
     */
    private Date birthdayDate;

    /**
     * 身份证生效日
     */
    private Date icValidityBegin;

    /**
     * 身份证到期日
     */
    private Date icValidityEnd;
    private String threeApplyStatus;

    /**
     * 等待放款时间
     */
    private Date waitLoanTime;

    /**
     * 回款时间
     */
    private Date paymentTime;

    /**
     * 国网授权状态（0：未导入 1: 已导入 2：已授权）
     */
    private String ifStateGridStatus;

    /**
     * 南网电力公司(手输)
     */
    private String provinceCodeHand;

    /**
     * 合同视频
     */
    private String contractVideoUrl;

    /**
     * 合同视频名称
     */
    private String contractVideoName;

    //自审结果（用于小程序）
    private String zsFlag;  //审核结果 1通过 2驳回
    private String nodeType; //驳回节点 1勘测 2设计
    private String zsAuditRemarks;

    /**
     * 竣工状态 0：未提交 1：已提交 2：已竣工
     */
    private String isCheckAccept;

    private String isSendMergeMsg; //是否发送并网短信 0.未发送 1.已发送

    private String addressFlag;//0.需要省市区 1.不需要省市区  只针对广发

    /**
     * 资方审查补充资料
     */
    private String otherAttachment;

    /**
     * 拆除状态 0：未拆除 1：已拆除
     */
    private String removeSign;

    private String  qrCodeUrl;//广发二维码
    private String windPressure;//风压
    private String snowPressure;//雪压
    private String pressureLevel;//风雪压等级
    private Date gridPushTime;

    public Date getGridPushTime() {
        return gridPushTime;
    }

    public void setGridPushTime(Date gridPushTime) {
        this.gridPushTime = gridPushTime;
    }

    public String getWindPressure() {
        return windPressure;
    }

    public void setWindPressure(String windPressure) {
        this.windPressure = windPressure;
    }

    public String getSnowPressure() {
        return snowPressure;
    }

    public void setSnowPressure(String snowPressure) {
        this.snowPressure = snowPressure;
    }

    public String getPressureLevel() {
        return pressureLevel;
    }

    public void setPressureLevel(String pressureLevel) {
        this.pressureLevel = pressureLevel;
    }

    public String getRemoveSign() {
        return removeSign;
    }

    public void setRemoveSign(String removeSign) {
        this.removeSign = removeSign;
    }

    public String getQrCodeUrl() {
        return qrCodeUrl;
    }

    public void setQrCodeUrl(String qrCodeUrl) {
        this.qrCodeUrl = qrCodeUrl;
    }

    public String getOtherAttachment() {
        return otherAttachment;
    }

    public void setOtherAttachment(String otherAttachment) {
        this.otherAttachment = otherAttachment;
    }

    public String getAddressFlag() {
        return addressFlag;
    }

    public void setAddressFlag(String addressFlag) {
        this.addressFlag = addressFlag;
    }

    public String getIsSendMergeMsg() {
        return isSendMergeMsg;
    }

    public void setIsSendMergeMsg(String isSendMergeMsg) {
        this.isSendMergeMsg = isSendMergeMsg;
    }

    public String getIsCheckAccept() {
        return isCheckAccept;
    }

    public void setIsCheckAccept(String isCheckAccept) {
        this.isCheckAccept = isCheckAccept;
    }

    public String getZsFlag() {
        return zsFlag;
    }

    public void setZsFlag(String zsFlag) {
        this.zsFlag = zsFlag;
    }

    public String getNodeType() {
        return nodeType;
    }

    public void setNodeType(String nodeType) {
        this.nodeType = nodeType;
    }

    public String getZsAuditRemarks() {
        return zsAuditRemarks;
    }

    public void setZsAuditRemarks(String zsAuditRemarks) {
        this.zsAuditRemarks = zsAuditRemarks;
    }

    public String getContractVideoUrl() {
        return contractVideoUrl;
    }

    public void setContractVideoUrl(String contractVideoUrl) {
        this.contractVideoUrl = contractVideoUrl;
    }

    public String getContractVideoName() {
        return contractVideoName;
    }

    public void setContractVideoName(String contractVideoName) {
        this.contractVideoName = contractVideoName;
    }

    public String getProvinceCodeHand() {
        return provinceCodeHand;
    }

    public void setProvinceCodeHand(String provinceCodeHand) {
        this.provinceCodeHand = provinceCodeHand;
    }

    public String getIfStateGridStatus() {
        return ifStateGridStatus;
    }

    public void setIfStateGridStatus(String ifStateGridStatus) {
        this.ifStateGridStatus = ifStateGridStatus;
    }

    public Date getWaitLoanTime() {
        return waitLoanTime;
    }

    public void setWaitLoanTime(Date waitLoanTime) {
        this.waitLoanTime = waitLoanTime;
    }

    public Date getPaymentTime() {
        return paymentTime;
    }

    public void setPaymentTime(Date paymentTime) {
        this.paymentTime = paymentTime;
    }

    private String isHistory; //历史电站标记：0.否 1.是

    public String getIsHistory() {
        return isHistory;
    }

    public void setIsHistory(String isHistory) {
        this.isHistory = isHistory;
    }

    public Date getBirthdayDate() {
        return birthdayDate;
    }

    public void setBirthdayDate(Date birthdayDate) {
        this.birthdayDate = birthdayDate;
    }

    public Date getIcValidityBegin() {
        return icValidityBegin;
    }

    public void setIcValidityBegin(Date icValidityBegin) {
        this.icValidityBegin = icValidityBegin;
    }

    public Date getIcValidityEnd() {
        return icValidityEnd;
    }

    public void setIcValidityEnd(Date icValidityEnd) {
        this.icValidityEnd = icValidityEnd;
    }

    public String getExigenceName() {
        return exigenceName;
    }

    public void setExigenceName(String exigenceName) {
        this.exigenceName = exigenceName;
    }

    public String getExigencePhone() {
        return exigencePhone;
    }

    public void setExigencePhone(String exigencePhone) {
        this.exigencePhone = exigencePhone;
    }

    public Date getInstallAcceptTime() {
        return installAcceptTime;
    }

    public void setInstallAcceptTime(Date installAcceptTime) {
        this.installAcceptTime = installAcceptTime;
    }

    public Date getInstallAuditFinishTime() {
        return installAuditFinishTime;
    }

    public void setInstallAuditFinishTime(Date installAuditFinishTime) {
        this.installAuditFinishTime = installAuditFinishTime;
    }

    public String getBwSettleFlag() {
        return bwSettleFlag;
    }

    public void setBwSettleFlag(String bwSettleFlag) {
        this.bwSettleFlag = bwSettleFlag;
    }

    public String getWgSettleFlag() {
        return wgSettleFlag;
    }

    public void setWgSettleFlag(String wgSettleFlag) {
        this.wgSettleFlag = wgSettleFlag;
    }

    public Date getFirstGenerationTime() {
        return firstGenerationTime;
    }

    public void setFirstGenerationTime(Date firstGenerationTime) {
        this.firstGenerationTime = firstGenerationTime;
    }

    public Date getInComingTime() {
        return inComingTime;
    }

    public void setInComingTime(Date inComingTime) {
        this.inComingTime = inComingTime;
    }

    public Date getMaterialFixFinishTime() {
        return materialFixFinishTime;
    }

    public void setMaterialFixFinishTime(Date materialFixFinishTime) {
        this.materialFixFinishTime = materialFixFinishTime;
    }

    public Date getPreAuditTime() {
        return preAuditTime;
    }

    public void setPreAuditTime(Date preAuditTime) {
        this.preAuditTime = preAuditTime;
    }

    public String getNbqDzId() {
        return nbqDzId;
    }

    public void setNbqDzId(String nbqDzId) {
        this.nbqDzId = nbqDzId;
    }

    public String getNbqFactory() {
        return nbqFactory;
    }

    public void setNbqFactory(String nbqFactory) {
        this.nbqFactory = nbqFactory;
    }

    public String getNbqSn() {
        return nbqSn;
    }

    public void setNbqSn(String nbqSn) {
        this.nbqSn = nbqSn;
    }

    public String getToExamineStatus() {
        return toExamineStatus;
    }

    public void setToExamineStatus(String toExamineStatus) {
        this.toExamineStatus = toExamineStatus;
    }

    public String getElectricProvince() {
        return electricProvince;
    }

    public void setElectricProvince(String electricProvince) {
        this.electricProvince = electricProvince;
    }

    public String getElectricCity() {
        return electricCity;
    }

    public void setElectricCity(String electricCity) {
        this.electricCity = electricCity;
    }

    public String getElectricArea() {
        return electricArea;
    }

    public void setElectricArea(String electricArea) {
        this.electricArea = electricArea;
    }

    public String getElectricAddress() {
        return electricAddress;
    }

    public void setElectricAddress(String electricAddress) {
        this.electricAddress = electricAddress;
    }

    /**
     * 电器组件
     */
    @Transient
    private List<CustomerEntryComponentVo> itemList;

    public String getEncryptionIdCard() {
        return encryptionIdCard;
    }

    public void setEncryptionIdCard(String encryptionIdCard) {
        this.encryptionIdCard = encryptionIdCard;
    }

    public String getGdtRemarks() {
        return gdtRemarks;
    }

    public void setGdtRemarks(String gdtRemarks) {
        this.gdtRemarks = gdtRemarks;
    }

    public Date getDispatchTime() {
        return dispatchTime;
    }

    public void setDispatchTime(Date dispatchTime) {
        this.dispatchTime = dispatchTime;
    }

    //状态 1.待审核 2.审核不通过 3.待下单 4.待收货 5.待安装 6.待并网 7.待验收 8.待支付 9.待归档 a.已完成
    public CustomerEntry() {
        super();
        this.status = "1";
        this.createTime = new Date();
        this.uniformity="1";
        this.schemeType="1";
        this.designCapacity=0;
        this.actualCapacity=BigDecimal.ZERO;
        this.retrial="1";
        this.whetherToPay ="1";
        this.applyChange = "1";
        this.acceptFlag="0";
        this.admitFlag = "0";
        this.dispatchStatus = "0";
        this.epcPrice = BigDecimal.ZERO;
        this.toExamineStatus="1";
        this.credit="0";
        this.paymentStatus = "0";
        this.authorization="1";
        this.secCardStatus="0";
        this.designAuditFlag = "0";
        this.installFlag = "0";
        this.checkAcceptFlag = "0";
        this.mergeFlag = "0";
        this.onGridFlag = "0";
        this.ifRewardSettle="0";
        this.flag = "0";
        this.zsResult = "0";
        this.nbqFactory = "00";
        this.nbqDzId = "";
        this.isHistory="0";
        this.bwSettleFlag="0";
        this.wgSettleFlag="0";
        this.isSendMergeMsg = "0";
        this.addressFlag = "1";
        this.removeSign = "0";
    }

    public void setProjectNum(int projectNum) {
        this.projectNum = projectNum;
    }

    public int getProjectNum() {
        return projectNum;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getProjectId() {
        return projectId;
    }

    public void setProjectId(String projectId) {
        this.projectId = projectId;
    }

    public String getProjectName() {
        return projectName;
    }

    public void setProjectName(String projectName) {
        this.projectName = projectName;
    }

    public String getProjectCode() {
        return projectCode;
    }

    public void setProjectCode(String projectCode) {
        this.projectCode = projectCode;
    }

    public String getDistributorId() {
        return distributorId;
    }

    public void setDistributorId(String distributorId) {
        this.distributorId = distributorId;
    }

    public String getTraderName() {
        return traderName;
    }

    public void setTraderName(String traderName) {
        this.traderName = traderName;
    }

    public String getTraderTel() {
        return traderTel;
    }

    public void setTraderTel(String traderTel) {
        this.traderTel = traderTel;
    }

    public String getPowerStationCode() {
        return powerStationCode;
    }

    public void setPowerStationCode(String powerStationCode) {
        this.powerStationCode = powerStationCode;
    }

    public String getTenantName() {
        return tenantName;
    }

    public void setTenantName(String tenantName) {
        this.tenantName = tenantName;
    }

    public String getProvince() {
        return province;
    }

    public void setProvince(String province) {
        this.province = province;
    }

    public String getCity() {
        return city;
    }

    public void setCity(String city) {
        this.city = city;
    }

    public String getArea() {
        return area;
    }

    public void setArea(String area) {
        this.area = area;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public BigDecimal getLongitude() {
        return longitude == null ? longitude : longitude.stripTrailingZeros();
    }

    public void setLongitude(BigDecimal longitude) {
        this.longitude = longitude;
    }

    public BigDecimal getLatitude() {
        return latitude == null ? latitude : latitude.stripTrailingZeros();
    }

    public void setLatitude(BigDecimal latitude) {
        this.latitude = latitude;
    }

    public String getMobile() {
        return mobile;
    }

    public void setMobile(String mobile) {
        this.mobile = mobile;
    }

    public String getIdCode() {
        return idCode;
    }

    public void setIdCode(String idCode) {
        this.idCode = idCode;
    }

    public String getRemarks() {
        return remarks;
    }

    public void setRemarks(String remarks) {
        this.remarks = remarks;
    }

    public String getIdCardFront() {
        return idCardFront;
    }

    public void setIdCardFront(String idCardFront) {
        this.idCardFront = idCardFront;
    }

    public String getIdCardBack() {
        return idCardBack;
    }

    public void setIdCardBack(String idCardBack) {
        this.idCardBack = idCardBack;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getApplyStatus() {
        return applyStatus;
    }

    public void setApplyStatus(String applyStatus) {
        this.applyStatus = applyStatus;
    }

    public String getAdmitFlag() {
        return admitFlag;
    }

    public void setAdmitFlag(String admitFlag) {
        this.admitFlag = admitFlag;
    }

    public String getCreateBy() {
        return createBy;
    }

    public void setCreateBy(String createBy) {
        this.createBy = createBy;
    }

    public String getCreator() {
        return creator;
    }

    public void setCreator(String creator) {
        this.creator = creator;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public String getSchemeType() {
        return schemeType;
    }

    public void setSchemeType(String schemeType) {
        this.schemeType = schemeType;
    }

    public String getSchemeTypeId() {
        return schemeTypeId;
    }

    public void setSchemeTypeId(String schemeTypeId) {
        this.schemeTypeId = schemeTypeId;
    }

    public String getSchemeTypeName() {
        return schemeTypeName;
    }

    public void setSchemeTypeName(String schemeTypeName) {
        this.schemeTypeName = schemeTypeName;
    }

    public String getSchemeId() {
        return schemeId;
    }

    public void setSchemeId(String schemeId) {
        this.schemeId = schemeId;
    }

    public String getSchemeName() {
        return schemeName;
    }

    public void setSchemeName(String schemeName) {
        this.schemeName = schemeName;
    }

    public String getDesignDrawings() {
        return designDrawings;
    }

    public void setDesignDrawings(String designDrawings) {
        this.designDrawings = designDrawings;
    }

    public String getDesignDrawingsName() {
        return designDrawingsName;
    }

    public void setDesignDrawingsName(String designDrawingsName) {
        this.designDrawingsName = designDrawingsName;
    }

    public String getReceivingReportId() {
        return receivingReportId;
    }

    public void setReceivingReportId(String receivingReportId) {
        this.receivingReportId = receivingReportId;
    }

    public Date getToExamineTime() {
        return toExamineTime;
    }

    public void setToExamineTime(Date toExamineTime) {
        this.toExamineTime = toExamineTime;
    }

    public String getExamine() {
        return examine;
    }

    public void setExamine(String examine) {
        this.examine = examine;
    }

    public String getExamineName() {
        return examineName;
    }

    public void setExamineName(String examineName) {
        this.examineName = examineName;
    }

    public Date getExamineTime() {
        return examineTime;
    }

    public void setExamineTime(Date examineTime) {
        this.examineTime = examineTime;
    }

    public String getExamineRemarks() {
        return examineRemarks;
    }

    public void setExamineRemarks(String examineRemarks) {
        this.examineRemarks = examineRemarks;
    }

    public Date getOrderTime() {
        return orderTime;
    }

    public void setOrderTime(Date orderTime) {
        this.orderTime = orderTime;
    }

    public String getOrderBy() {
        return orderBy;
    }

    public void setOrderBy(String orderBy) {
        this.orderBy = orderBy;
    }

    public String getOrderName() {
        return orderName;
    }

    public void setOrderName(String orderName) {
        this.orderName = orderName;
    }

    public String getReceiveBy() {
        return receiveBy;
    }

    public void setReceiveBy(String receiveBy) {
        this.receiveBy = receiveBy;
    }

    public Date getReceiveTime() {
        return receiveTime;
    }

    public void setReceiveTime(Date receiveTime) {
        this.receiveTime = receiveTime;
    }

    public String getReceiveName() {
        return receiveName;
    }

    public void setReceiveName(String receiveName) {
        this.receiveName = receiveName;
    }

    public String getDispatchStatus() {
        return dispatchStatus;
    }

    public void setDispatchStatus(String dispatchStatus) {
        this.dispatchStatus = dispatchStatus;
    }

    public String getEngineerName() {
        return engineerName;
    }

    public void setEngineerName(String engineerName) {
        this.engineerName = engineerName;
    }

    public String getEngineerId() {
        return engineerId;
    }

    public void setEngineerId(String engineerId) {
        this.engineerId = engineerId;
    }

    public String getInstallBy() {
        return installBy;
    }

    public void setInstallBy(String installBy) {
        this.installBy = installBy;
    }

    public Date getInstallTime() {
        return installTime;
    }

    public void setInstallTime(Date installTime) {
        this.installTime = installTime;
    }

    public String getMergeBy() {
        return mergeBy;
    }

    public void setMergeBy(String mergeBy) {
        this.mergeBy = mergeBy;
    }

    public String getMergeName() {
        return mergeName;
    }

    public void setMergeName(String mergeName) {
        this.mergeName = mergeName;
    }

    public Date getMergeTime() {
        return mergeTime;
    }

    public void setMergeTime(Date mergeTime) {
        this.mergeTime = mergeTime;
    }

    public String getCheckAcceptBy() {
        return checkAcceptBy;
    }

    public void setCheckAcceptBy(String checkAcceptBy) {
        this.checkAcceptBy = checkAcceptBy;
    }

    public String getCheckAcceptName() {
        return checkAcceptName;
    }

    public void setCheckAcceptName(String checkAcceptName) {
        this.checkAcceptName = checkAcceptName;
    }

    public Date getCheckAcceptTime() {
        return checkAcceptTime;
    }

    public void setCheckAcceptTime(Date checkAcceptTime) {
        this.checkAcceptTime = checkAcceptTime;
    }

    public String getCheckAcceptImgs() {
        return checkAcceptImgs;
    }

    public void setCheckAcceptImgs(String checkAcceptImgs) {
        this.checkAcceptImgs = checkAcceptImgs;
    }

    public String getDeliverBy() {
        return deliverBy;
    }

    public void setDeliverBy(String deliverBy) {
        this.deliverBy = deliverBy;
    }

    public String getDeliverName() {
        return deliverName;
    }

    public void setDeliverName(String deliverName) {
        this.deliverName = deliverName;
    }

    public Date getDeliverTime() {
        return deliverTime;
    }

    public void setDeliverTime(Date deliverTime) {
        this.deliverTime = deliverTime;
    }

    public String getDeliverFile() {
        return deliverFile;
    }

    public void setDeliverFile(String deliverFile) {
        this.deliverFile = deliverFile;
    }

    public String getIndocBy() {
        return indocBy;
    }

    public void setIndocBy(String indocBy) {
        this.indocBy = indocBy;
    }

    public String getIndocName() {
        return indocName;
    }

    public void setIndocName(String indocName) {
        this.indocName = indocName;
    }

    public Date getIndocTime() {
        return indocTime;
    }

    public void setIndocTime(Date indocTime) {
        this.indocTime = indocTime;
    }

    public String getFlag() {
        return flag;
    }

    public void setFlag(String flag) {
        this.flag = flag;
    }

    public String getUniformity() {
        return uniformity;
    }

    public void setUniformity(String uniformity) {
        this.uniformity = uniformity;
    }

    public String getComponentSpec() {
        return componentSpec;
    }

    public void setComponentSpec(String componentSpec) {
        this.componentSpec = componentSpec;
    }

    public Integer getDesignCapacity() {
        return designCapacity;
    }

    public void setDesignCapacity(Integer designCapacity) {
        this.designCapacity = designCapacity;
    }

    public BigDecimal getActualCapacity() {
        return actualCapacity;
    }

    public void setActualCapacity(BigDecimal actualCapacity) {
        this.actualCapacity = actualCapacity;
    }

    public String getSchemeRemarks() {
        return schemeRemarks;
    }

    public void setSchemeRemarks(String schemeRemarks) {
        this.schemeRemarks = schemeRemarks;
    }

    public String getGeneratorNo() {
        return generatorNo;
    }

    public void setGeneratorNo(String generatorNo) {
        this.generatorNo = generatorNo;
    }

    public String getGridInstitution() {
        return gridInstitution;
    }

    public void setGridInstitution(String gridInstitution) {
        this.gridInstitution = gridInstitution;
    }

    public String getPowerPlantCode() {
        return powerPlantCode;
    }

    public void setPowerPlantCode(String powerPlantCode) {
        this.powerPlantCode = powerPlantCode;
    }

    public String getProvinceCode() {
        return provinceCode;
    }

    public void setProvinceCode(String provinceCode) {
        this.provinceCode = provinceCode;
    }

    public String getWhetherToPay() {
        return whetherToPay;
    }

    public void setWhetherToPay(String whetherToPay) {
        this.whetherToPay = whetherToPay;
    }

    public String getRetrial() {
        return retrial;
    }

    public void setRetrial(String retrial) {
        this.retrial = retrial;
    }

    public String getApplyReasonId() {
        return applyReasonId;
    }

    public void setApplyReasonId(String applyReasonId) {
        this.applyReasonId = applyReasonId;
    }

    public String getApplyReasonName() {
        return applyReasonName;
    }

    public void setApplyReasonName(String applyReasonName) {
        this.applyReasonName = applyReasonName;
    }

    public Date getApplyTime() {
        return applyTime;
    }

    public void setApplyTime(Date applyTime) {
        this.applyTime = applyTime;
    }

    public String getApplyChange() {
        return applyChange;
    }

    public void setApplyChange(String applyChange) {
        this.applyChange = applyChange;
    }

    public String getRectificationSetId() {
        return rectificationSetId;
    }

    public void setRectificationSetId(String rectificationSetId) {
        this.rectificationSetId = rectificationSetId;
    }

    public String getChangeReason() {
        return changeReason;
    }

    public void setChangeReason(String changeReason) {
        this.changeReason = changeReason;
    }

    public String getAuditComments() {
        return auditComments;
    }

    public void setAuditComments(String auditComments) {
        this.auditComments = auditComments;
    }

    public Date getChangeTime() {
        return changeTime;
    }

    public void setChangeTime(Date changeTime) {
        this.changeTime = changeTime;
    }

    public String getAcceptFlag() {
        return acceptFlag;
    }

    public void setAcceptFlag(String acceptFlag) {
        this.acceptFlag = acceptFlag;
    }

    public Date getAcceptTime() {
        return acceptTime;
    }

    public void setAcceptTime(Date acceptTime) {
        this.acceptTime = acceptTime;
    }

    public String getAcceptBy() {
        return acceptBy;
    }

    public void setAcceptBy(String acceptBy) {
        this.acceptBy = acceptBy;
    }

    public String getAcceptor() {
        return acceptor;
    }

    public void setAcceptor(String acceptor) {
        this.acceptor = acceptor;
    }

    public String getPartnerId() {
        return partnerId;
    }

    public void setPartnerId(String partnerId) {
        this.partnerId = partnerId;
    }

    public String getPartnerName() {
        return partnerName;
    }

    public void setPartnerName(String partnerName) {
        this.partnerName = partnerName;
    }

    public String getManagementType() {
        return managementType;
    }

    public void setManagementType(String managementType) {
        this.managementType = managementType;
    }

    public String getCredit() {
        return credit;
    }

    public void setCredit(String credit) {
        this.credit = credit;
    }

    public String getAuthorization() {
        return authorization;
    }

    public void setAuthorization(String authorization) {
        this.authorization = authorization;
    }

    public String getCreditDsc() {
        return creditDsc;
    }

    public void setCreditDsc(String creditDsc) {
        this.creditDsc = creditDsc;
    }

    public String getFirstApplyStatus() {
        return firstApplyStatus;
    }

    public void setFirstApplyStatus(String firstApplyStatus) {
        this.firstApplyStatus = firstApplyStatus;
    }

    public String getNextApplyStatus() {
        return nextApplyStatus;
    }

    public void setNextApplyStatus(String nextApplyStatus) {
        this.nextApplyStatus = nextApplyStatus;
    }

    public String getApplyBackReason() {
        return applyBackReason;
    }

    public void setApplyBackReason(String applyBackReason) {
        this.applyBackReason = applyBackReason;
    }

    public Date getApplyBackTime() {
        return applyBackTime;
    }

    public void setApplyBackTime(Date applyBackTime) {
        this.applyBackTime = applyBackTime;
    }

    public List<ComposePlan> getComposePlanLists() {
        return composePlanLists;
    }

    public void setComposePlanLists(List<ComposePlan> composePlanLists) {
        this.composePlanLists = composePlanLists;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getContract() {
        return contract;
    }

    public void setContract(String contract) {
        this.contract = contract;
    }

    public List<CustomerEntryComponentVo> getItemList() {
        return itemList;
    }

    public void setItemList(List<CustomerEntryComponentVo> itemList) {
        this.itemList = itemList;
    }

    public String getBill() {
        return bill;
    }

    public void setBill(String bill) {
        this.bill = bill;
    }

    public String getBillNumber() {
        return billNumber;
    }

    public void setBillNumber(String billNumber) {
        this.billNumber = billNumber;
    }

    public String getOneselfFlag() {
        return oneselfFlag;
    }

    public void setOneselfFlag(String oneselfFlag) {
        this.oneselfFlag = oneselfFlag;
    }

    public BigDecimal getEpcPrice() {
        return epcPrice;
    }

    public void setEpcPrice(BigDecimal epcPrice) {
        this.epcPrice = epcPrice;
    }

    public String getProjectCompanyId() {
        return projectCompanyId;
    }

    public void setProjectCompanyId(String projectCompanyId) {
        this.projectCompanyId = projectCompanyId;
    }

    public String getProjectCompanyName() {
        return projectCompanyName;
    }

    public void setProjectCompanyName(String projectCompanyName) {
        this.projectCompanyName = projectCompanyName;
    }

    public String getProductLineId() {
        return productLineId;
    }

    public void setProductLineId(String productLineId) {
        this.productLineId = productLineId;
    }

    public String getProductLineName() {
        return productLineName;
    }

    public void setProductLineName(String productLineName) {
        this.productLineName = productLineName;
    }

    public String getProductLine() {
        return productLine;
    }

    public void setProductLine(String productLine) {
        this.productLine = productLine;
    }

    public String getPayCycle() {
        return payCycle;
    }

    public void setPayCycle(String payCycle) {
        this.payCycle = payCycle;
    }

    public String getSignStatus() {
        return signStatus;
    }

    public void setSignStatus(String signStatus) {
        this.signStatus = signStatus;
    }

    public String getPaymentStatus() {
        return paymentStatus;
    }

    public void setPaymentStatus(String paymentStatus) {
        this.paymentStatus = paymentStatus;
    }

    public String getSn() {
        return sn;
    }

    public void setSn(String sn) {
        this.sn = sn;
    }

    public BigDecimal getEtd() {
        return etd;
    }

    public void setEtd(BigDecimal etd) {
        this.etd = etd;
    }

    public BigDecimal getEto() {
        return eto;
    }

    public void setEto(BigDecimal eto) {
        this.eto = eto;
    }

    public BigDecimal getEtdc() {
        return etdc;
    }

    public void setEtdc(BigDecimal etdc) {
        this.etdc = etdc;
    }

    public BigDecimal getEtoc() {
        return etoc;
    }

    public void setEtoc(BigDecimal etoc) {
        this.etoc = etoc;
    }

    public String getWarehouseId() {
        return warehouseId;
    }

    public void setWarehouseId(String warehouseId) {
        this.warehouseId = warehouseId;
    }

    public String getBankName() {
        return bankName;
    }

    public void setBankName(String bankName) {
        this.bankName = bankName;
    }

    public String getBankCode() {
        return bankCode;
    }

    public void setBankCode(String bankCode) {
        this.bankCode = bankCode;
    }

    public String getStreet() {
        return street;
    }

    public void setStreet(String street) {
        this.street = street;
    }

    public String getGender() {
        return gender;
    }

    public void setGender(String gender) {
        this.gender = gender;
    }

    public Integer getAge() {
        return age;
    }

    public void setAge(Integer age) {
        this.age = age;
    }

    public String getResidenceBooklet() {
        return residenceBooklet;
    }

    public void setResidenceBooklet(String residenceBooklet) {
        this.residenceBooklet = residenceBooklet;
    }

    public String getTssBankNum() {
        return tssBankNum;
    }

    public void setTssBankNum(String tssBankNum) {
        this.tssBankNum = tssBankNum;
    }

    public String getBankCard() {
        return bankCard;
    }

    public void setBankCard(String bankCard) {
        this.bankCard = bankCard;
    }

    public String getInsuranceFlag() {
        return insuranceFlag;
    }

    public void setInsuranceFlag(String insuranceFlag) {
        this.insuranceFlag = insuranceFlag;
    }

    public Date getLeaseDate() {
        return leaseDate;
    }

    public void setLeaseDate(Date leaseDate) {
        this.leaseDate = leaseDate;
    }

    public String getBpName() {
        return bpName;
    }

    public void setBpName(String bpName) {
        this.bpName = bpName;
    }

    public String getBpRelation() {
        return bpRelation;
    }

    public void setBpRelation(String bpRelation) {
        this.bpRelation = bpRelation;
    }

    public String getBpIdCard() {
        return bpIdCard;
    }

    public void setBpIdCard(String bpIdCard) {
        this.bpIdCard = bpIdCard;
    }

    public String getBpMobile() {
        return bpMobile;
    }

    public void setBpMobile(String bpMobile) {
        this.bpMobile = bpMobile;
    }

    public String getBpCardFront() {
        return bpCardFront;
    }

    public void setBpCardFront(String bpCardFront) {
        this.bpCardFront = bpCardFront;
    }

    public String getBpCardBack() {
        return bpCardBack;
    }

    public void setBpCardBack(String bpCardBack) {
        this.bpCardBack = bpCardBack;
    }

    public String getBpFile() {
        return bpFile;
    }

    public void setBpFile(String bpFile) {
        this.bpFile = bpFile;
    }

    public String getZjpbImg() {
        return zjpbImg;
    }

    public void setZjpbImg(String zjpbImg) {
        this.zjpbImg = zjpbImg;
    }

    public String getZjzxImg() {
        return zjzxImg;
    }

    public void setZjzxImg(String zjzxImg) {
        this.zjzxImg = zjzxImg;
    }

    public String getZjcsImg() {
        return zjcsImg;
    }

    public void setZjcsImg(String zjcsImg) {
        this.zjcsImg = zjcsImg;
    }

    public String getZjhsImg() {
        return zjhsImg;
    }

    public void setZjhsImg(String zjhsImg) {
        this.zjhsImg = zjhsImg;
    }

    public String getNbqwzImg() {
        return nbqwzImg;
    }

    public void setNbqwzImg(String nbqwzImg) {
        this.nbqwzImg = nbqwzImg;
    }

    public String getPdxwzImg() {
        return pdxwzImg;
    }

    public void setPdxwzImg(String pdxwzImg) {
        this.pdxwzImg = pdxwzImg;
    }

    public String getFilingType() {
        return filingType;
    }

    public void setFilingType(String filingType) {
        this.filingType = filingType;
    }

    public String getSecCardStatus() {
        return secCardStatus;
    }

    public void setSecCardStatus(String secCardStatus) {
        this.secCardStatus = secCardStatus;
    }

    public String getAddressConsistent() {
        return addressConsistent;
    }

    public void setAddressConsistent(String addressConsistent) {
        this.addressConsistent = addressConsistent;
    }

    public String getCreateTraderId() {
        return createTraderId;
    }

    public void setCreateTraderId(String createTraderId) {
        this.createTraderId = createTraderId;
    }

    public String getCreateTraderName() {
        return createTraderName;
    }

    public void setCreateTraderName(String createTraderName) {
        this.createTraderName = createTraderName;
    }

    public String getTssBankNumName() {
        return tssBankNumName;
    }

    public void setTssBankNumName(String tssBankNumName) {
        this.tssBankNumName = tssBankNumName;
    }

    public String getDesignAuditFlag() {
        return designAuditFlag;
    }

    public void setDesignAuditFlag(String designAuditFlag) {
        this.designAuditFlag = designAuditFlag;
    }

    public String getInstallFlag() {
        return installFlag;
    }

    public void setInstallFlag(String installFlag) {
        this.installFlag = installFlag;
    }

    public String getCheckAcceptFlag() {
        return checkAcceptFlag;
    }

    public void setCheckAcceptFlag(String checkAcceptFlag) {
        this.checkAcceptFlag = checkAcceptFlag;
    }

    public String getMergeFlag() {
        return mergeFlag;
    }

    public void setMergeFlag(String mergeFlag) {
        this.mergeFlag = mergeFlag;
    }

    public String getContractFilename() {
        return contractFilename;
    }

    public void setContractFilename(String contractFilename) {
        this.contractFilename = contractFilename;
    }

    public String getTenantId() {
        return tenantId;
    }

    public void setTenantId(String tenantId) {
        this.tenantId = tenantId;
    }

    public String getTenantType() {
        return tenantType;
    }

    public void setTenantType(String tenantType) {
        this.tenantType = tenantType;
    }

    public String getCorp() {
        return corp;
    }

    public void setCorp(String corp) {
        this.corp = corp;
    }

    public String getRegno() {
        return regno;
    }

    public void setRegno(String regno) {
        this.regno = regno;
    }

    public String getRoofType() {
        return roofType;
    }

    public void setRoofType(String roofType) {
        this.roofType = roofType;
    }

    public String getMaterialFixFlag() {
        return materialFixFlag;
    }

    public void setMaterialFixFlag(String materialFixFlag) {
        this.materialFixFlag = materialFixFlag;
    }

    public String getMaterialMergeFlag() {
        return materialMergeFlag;
    }

    public void setMaterialMergeFlag(String materialMergeFlag) {
        this.materialMergeFlag = materialMergeFlag;
    }

    public String getAuditRemarks() {
        return auditRemarks;
    }

    public void setAuditRemarks(String auditRemarks) {
        this.auditRemarks = auditRemarks;
    }

    public String getDzImgs() {
        return dzImgs;
    }

    public void setDzImgs(String dzImgs) {
        this.dzImgs = dzImgs;
    }

    public String getIfRewardSettle() {
        return ifRewardSettle;
    }

    public void setIfRewardSettle(String ifRewardSettle) {
        this.ifRewardSettle = ifRewardSettle;
    }

    public String getZsResult() {
        return zsResult;
    }

    public void setZsResult(String zsResult) {
        this.zsResult = zsResult;
    }

    @Override
    public String toString() {
        return "CustomerEntry{" +
                "id='" + id + '\'' +
                ", projectId='" + projectId + '\'' +
                ", projectName='" + projectName + '\'' +
                ", projectCode='" + projectCode + '\'' +
                ", distributorId='" + distributorId + '\'' +
                ", traderName='" + traderName + '\'' +
                ", traderTel='" + traderTel + '\'' +
                ", powerStationCode='" + powerStationCode + '\'' +
                ", tenantId='" + tenantId + '\'' +
                ", tenantType='" + tenantType + '\'' +
                ", tenantName='" + tenantName + '\'' +
                ", province='" + province + '\'' +
                ", city='" + city + '\'' +
                ", area='" + area + '\'' +
                ", address='" + address + '\'' +
                ", street='" + street + '\'' +
                ", longitude=" + longitude +
                ", latitude=" + latitude +
                ", mobile='" + mobile + '\'' +
                ", idCode='" + idCode + '\'' +
                ", remarks='" + remarks + '\'' +
                ", encryptionIdCard='" + encryptionIdCard + '\'' +
                ", idCardFront='" + idCardFront + '\'' +
                ", idCardBack='" + idCardBack + '\'' +
                ", status='" + status + '\'' +
                ", toExamineStatus='" + toExamineStatus + '\'' +
                ", applyStatus='" + applyStatus + '\'' +
                ", admitFlag='" + admitFlag + '\'' +
                ", createBy='" + createBy + '\'' +
                ", creator='" + creator + '\'' +
                ", createTime=" + createTime +
                ", schemeType='" + schemeType + '\'' +
                ", schemeTypeId='" + schemeTypeId + '\'' +
                ", schemeTypeName='" + schemeTypeName + '\'' +
                ", schemeId='" + schemeId + '\'' +
                ", schemeName='" + schemeName + '\'' +
                ", designDrawings='" + designDrawings + '\'' +
                ", designDrawingsName='" + designDrawingsName + '\'' +
                ", receivingReportId='" + receivingReportId + '\'' +
                ", toExamineTime=" + toExamineTime +
                ", examine='" + examine + '\'' +
                ", examineName='" + examineName + '\'' +
                ", examineTime=" + examineTime +
                ", examineRemarks='" + examineRemarks + '\'' +
                ", orderTime=" + orderTime +
                ", orderBy='" + orderBy + '\'' +
                ", orderName='" + orderName + '\'' +
                ", receiveBy='" + receiveBy + '\'' +
                ", receiveTime=" + receiveTime +
                ", receiveName='" + receiveName + '\'' +
                ", dispatchStatus='" + dispatchStatus + '\'' +
                ", dispatchTime=" + dispatchTime +
                ", engineerName='" + engineerName + '\'' +
                ", engineerId='" + engineerId + '\'' +
                ", installBy='" + installBy + '\'' +
                ", installTime=" + installTime +
                ", mergeBy='" + mergeBy + '\'' +
                ", mergeName='" + mergeName + '\'' +
                ", mergeTime=" + mergeTime +
                ", checkAcceptBy='" + checkAcceptBy + '\'' +
                ", checkAcceptName='" + checkAcceptName + '\'' +
                ", checkAcceptTime=" + checkAcceptTime +
                ", checkAcceptImgs='" + checkAcceptImgs + '\'' +
                ", deliverBy='" + deliverBy + '\'' +
                ", deliverName='" + deliverName + '\'' +
                ", deliverTime=" + deliverTime +
                ", deliverFile='" + deliverFile + '\'' +
                ", indocBy='" + indocBy + '\'' +
                ", indocName='" + indocName + '\'' +
                ", indocTime=" + indocTime +
                ", flag='" + flag + '\'' +
                ", uniformity='" + uniformity + '\'' +
                ", componentSpec='" + componentSpec + '\'' +
                ", designCapacity=" + designCapacity +
                ", actualCapacity=" + actualCapacity +
                ", schemeRemarks='" + schemeRemarks + '\'' +
                ", generatorNo='" + generatorNo + '\'' +
                ", provinceCode='" + provinceCode + '\'' +
                ", whetherToPay='" + whetherToPay + '\'' +
                ", retrial='" + retrial + '\'' +
                ", applyReasonId='" + applyReasonId + '\'' +
                ", applyReasonName='" + applyReasonName + '\'' +
                ", applyTime=" + applyTime +
                ", applyChange='" + applyChange + '\'' +
                ", rectificationSetId='" + rectificationSetId + '\'' +
                ", changeReason='" + changeReason + '\'' +
                ", auditComments='" + auditComments + '\'' +
                ", changeTime=" + changeTime +
                ", acceptFlag='" + acceptFlag + '\'' +
                ", acceptTime=" + acceptTime +
                ", acceptBy='" + acceptBy + '\'' +
                ", acceptor='" + acceptor + '\'' +
                ", partnerId='" + partnerId + '\'' +
                ", partnerName='" + partnerName + '\'' +
                ", managementType='" + managementType + '\'' +
                ", credit='" + credit + '\'' +
                ", authorization='" + authorization + '\'' +
                ", creditDsc='" + creditDsc + '\'' +
                ", firstApplyStatus='" + firstApplyStatus + '\'' +
                ", nextApplyStatus='" + nextApplyStatus + '\'' +
                ", applyBackReason='" + applyBackReason + '\'' +
                ", contract='" + contract + '\'' +
                ", contractFilename='" + contractFilename + '\'' +
                ", bill='" + bill + '\'' +
                ", billNumber='" + billNumber + '\'' +
                ", oneselfFlag='" + oneselfFlag + '\'' +
                ", regionId='" + regionId + '\'' +
                ", epcPrice=" + epcPrice +
                ", projectCompanyId='" + projectCompanyId + '\'' +
                ", projectCompanyName='" + projectCompanyName + '\'' +
                ", productLineId='" + productLineId + '\'' +
                ", productLineName='" + productLineName + '\'' +
                ", productLine='" + productLine + '\'' +
                ", payCycle='" + payCycle + '\'' +
                ", signStatus='" + signStatus + '\'' +
                ", sn='" + sn + '\'' +
                ", nbqSn='" + nbqSn + '\'' +
                ", etd=" + etd +
                ", eto=" + eto +
                ", etdc=" + etdc +
                ", etoc=" + etoc +
                ", warehouseId='" + warehouseId + '\'' +
                ", gdtRemarks='" + gdtRemarks + '\'' +
                ", bankName='" + bankName + '\'' +
                ", bankCode='" + bankCode + '\'' +
                ", gender='" + gender + '\'' +
                ", age=" + age +
                ", residenceBooklet='" + residenceBooklet + '\'' +
                ", tssBankNum='" + tssBankNum + '\'' +
                ", tssBankNumName='" + tssBankNumName + '\'' +
                ", bankCard='" + bankCard + '\'' +
                ", insuranceFlag='" + insuranceFlag + '\'' +
                ", leaseDate=" + leaseDate +
                ", bpName='" + bpName + '\'' +
                ", bpIdCard='" + bpIdCard + '\'' +
                ", bpMobile='" + bpMobile + '\'' +
                ", bpCardFront='" + bpCardFront + '\'' +
                ", bpCardBack='" + bpCardBack + '\'' +
                ", bpFile='" + bpFile + '\'' +
                ", secCardStatus='" + secCardStatus + '\'' +
                ", zjpbImg='" + zjpbImg + '\'' +
                ", zjzxImg='" + zjzxImg + '\'' +
                ", zjcsImg='" + zjcsImg + '\'' +
                ", zjhsImg='" + zjhsImg + '\'' +
                ", nbqwzImg='" + nbqwzImg + '\'' +
                ", pdxwzImg='" + pdxwzImg + '\'' +
                ", filingType='" + filingType + '\'' +
                ", addressConsistent='" + addressConsistent + '\'' +
                ", electricProvince='" + electricProvince + '\'' +
                ", electricCity='" + electricCity + '\'' +
                ", electricArea='" + electricArea + '\'' +
                ", electricAddress='" + electricAddress + '\'' +
                ", createTraderId='" + createTraderId + '\'' +
                ", createTraderName='" + createTraderName + '\'' +
                ", designAuditFlag='" + designAuditFlag + '\'' +
                ", installFlag='" + installFlag + '\'' +
                ", checkAcceptFlag='" + checkAcceptFlag + '\'' +
                ", mergeFlag='" + mergeFlag + '\'' +
                ", onGridFlag='" + onGridFlag + '\'' +
                ", corp='" + corp + '\'' +
                ", regno='" + regno + '\'' +
                ", roofType='" + roofType + '\'' +
                ", materialFixFlag='" + materialFixFlag + '\'' +
                ", materialMergeFlag='" + materialMergeFlag + '\'' +
                ", composePlanLists=" + composePlanLists +
                ", type='" + type + '\'' +
                ", auditRemarks='" + auditRemarks + '\'' +
                ", itemList=" + itemList +
                ", zsResult=" + zsResult +
                '}';
    }

    public String getThreeApplyStatus() {
        return threeApplyStatus;
    }

    public void setThreeApplyStatus(String threeApplyStatus) {
        this.threeApplyStatus = threeApplyStatus;
    }
}
