package com.sifang.modules.flowpath.web;

import com.jfinal.plugin.activerecord.Record;
import com.sifang.modules.flowable.service.ProcessFlowableService;
import com.sifang.modules.flowable.utils.FormUtils;
import com.sifang.modules.flowpath.entity.ExamineInfo;
import com.sifang.modules.flowpath.service.DistributionGoodsApplyService;
import com.sifang.modules.flowpath.service.ExamineInfoService;
import com.sifang.modules.flowpath.vo.DistributionGoodsVo;
import com.sifang.modules.flowpath.vo.UserApplyVo;
import com.sifang.modules.sunshine.SystemSetting.utils.Result;
import com.sifang.modules.sys.common.UserUtils;
import com.sifang.modules.sys.common.entity.User;
import com.sifang.modules.sys.util.LayuiPage;
import com.sifang.modules.sys.web.BaseController;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

// 授信额调整申请
@Controller
@RequestMapping(value = "${adminPath}/apply/distributionGoods")
public class DistributionGoodsApplyController extends BaseController {

    protected Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private DistributionGoodsApplyService applyService;

    @Autowired
    private ExamineInfoService examineInfoService;

    @Autowired
    private ProcessFlowableService flowableService;

    //页面
    @RequestMapping(value = "applyListPage")
    public String applyListPage(Model model) {
        User user = UserUtils.getUser();
        model.addAttribute("userId",user.getId());
        model.addAttribute("formId", FormUtils.DISTRIBUTION_ADJUSTMENT_APPLICATION);
        return "modules/flowpath/distributionGoodsApplyListPage";
    }

    @ResponseBody
    @RequestMapping(value = "listData")
    public String listData(HttpServletRequest request){
        Map<String, String> map = getParams(request);
        LayuiPage page = new LayuiPage(request);
        applyService.applyList(page,map);
        return page.renderJson();
    }

    //新增页面
    @RequestMapping(value = "addDistributionGoodsPage")
    public String addDistributionGoodsPage(Model model){
       List<Record> list=applyService.getTrader();
       model.addAttribute("traders",list);
        model.addAttribute("imgsCount", 0);
        return "modules/flowpath/addDistributionGoodsPage";
    }

    //新建/修改申请
    @ResponseBody
    @RequestMapping(value = "saveDistributionGoods")
    public Result saveDistributionGoods(@RequestBody DistributionGoodsVo vo){
        try{
            applyService.saveDistributionGoods(vo);
            return Result.ok();
        }catch (RuntimeException r){
            r.printStackTrace();
            return Result.fail(r.getMessage());
        }catch (Exception e){
           e.printStackTrace();
          logger.error(">>>>>>>>>>>saveDistributionGoods e:{}",e);
        }
       return Result.fail("保存失败");
    }

    //编辑页面
    @RequestMapping(value = "editDistributionGoodsPage")
    public String editDistributionGoodsPage(HttpServletRequest request,Model model){
        String id=request.getParameter("id");
        Record apply=applyService.getDetailById(id);
        model.addAttribute("apply",apply);
        List<Record> list=applyService.getTrader();
        model.addAttribute("traders",list);
        List<ExamineInfo> infos=examineInfoService.getByApplyId(id,FormUtils.DISTRIBUTION_ADJUSTMENT_APPLICATION);
        model.addAttribute("infos",infos);
        int imgsCount = 0;
        if (StringUtils.isNotBlank(apply.getStr("voucher"))) {
            String[] imgArr = apply.getStr("voucher").split(",");
            imgsCount = imgArr.length;
            List<String> imglist = Arrays.asList(imgArr);
            model.addAttribute("imglist", imglist);
        }
        model.addAttribute("imgsCount", imgsCount);
        return "modules/flowpath/addDistributionGoodsPage";
    }

    //查看详情
    @RequestMapping(value = "ApplyDistributionGoods")
    public String ApplyDistributionGoods(HttpServletRequest request,Model model){
        String id=request.getParameter("id");
        String from=request.getParameter("from"); //from = audit 那么就是审批操作  否则就是查看
        model.addAttribute("from",from);
        Record apply=applyService.getDetailById(id);
        model.addAttribute("apply",apply);
        List<ExamineInfo> infos=examineInfoService.getByApplyId(id,FormUtils.DISTRIBUTION_ADJUSTMENT_APPLICATION);
        model.addAttribute("infos",infos);
        if (StringUtils.isNotBlank(apply.getStr("voucher"))) {
            String[] imgArr = apply.getStr("voucher").split(",");
            List<String> imglist = Arrays.asList(imgArr);
            model.addAttribute("imglist", imglist);
        }
        User user = UserUtils.getUser();
        model.addAttribute("userType",user.getUserType());
        if(!"audit".equals(from)){
            flowableService.readApplication(apply.getStr("process_instance_id"),FormUtils.DISTRIBUTION_ADJUSTMENT_APPLICATION,id,user);
        }
        return "modules/flowpath/applyDistributionGoodsDetail";
    }

    //删除申请
    @ResponseBody
    @RequestMapping(value = "deleteDistributionGoodsApply")
    public Result deleteDistributionGoodsApply(HttpServletRequest request){
        String id=request.getParameter("id");
        try{
            applyService.deleteDistributionGoodsApply(id);
            return Result.ok();
        }catch (RuntimeException r){
            r.printStackTrace();
            return Result.fail(r.getMessage());
        }catch (Exception e){
            logger.error(">>>>>>>>>>>>>>deleteDistributionGoodsApply e:{}",e);
            e.printStackTrace();
            return Result.fail("删除失败！");
        }
    }

    //审核通过
    @ResponseBody
    @RequestMapping(value = "examineDistributionGoodsApprove")
    public Result examineDistributionGoodsApprove(UserApplyVo userApplyVo){
        try{
            applyService.examineDistributionGoodsApprove(userApplyVo.getId(),userApplyVo.getTaskId(),userApplyVo.getContent());
            return Result.ok();
        }catch (Exception e){
            logger.error(">>>>>>>>>>>>>>examineDistributionGoodsApprove e:{}",e);
            return Result.fail("审核失败！");
        }
    }

    //审核驳回
    @ResponseBody
    @RequestMapping(value = "rejectTask")
    public Result rejectTask(@Valid UserApplyVo userApplyVo){
        try{
            applyService.rejectTask(userApplyVo.getId(),userApplyVo.getTaskId(),userApplyVo.getContent());
            return Result.ok();
        }catch (Exception e){
            logger.error(">>>>>>>>>>>>>>rejectTask e:{}",e);
            return Result.fail("审核失败！");
        }
    }



}
