package com.sifang.modules.flowpath.web;


import com.jfinal.plugin.activerecord.Record;
import com.sifang.modules.flowable.utils.FormUtils;
import com.sifang.modules.flowpath.entity.ExamineInfo;
import com.sifang.modules.flowpath.service.DistributionGoodsApplyService;
import com.sifang.modules.flowpath.service.ExamineInfoService;
import com.sifang.modules.flowpath.service.WarrantyDepositAdjustmentApplyService;
import com.sifang.modules.flowpath.vo.DistributionGoodsVo;
import com.sifang.modules.flowpath.vo.UserApplyVo;
import com.sifang.modules.sunshine.SystemSetting.utils.Result;
import com.sifang.modules.sys.common.UserUtils;
import com.sifang.modules.sys.common.entity.User;
import com.sifang.modules.sys.util.LayuiPage;
import com.sifang.modules.sys.web.BaseController;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

// 质保金调整申请
@Controller
@RequestMapping(value = "${adminPath}/apply/warrantyDeposit")
public class WarrantyDepositAdjustmentApplyController extends BaseController {

    protected Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WarrantyDepositAdjustmentApplyService warrantyApplyService;

    @Autowired
    private DistributionGoodsApplyService distributionGoodsApplyService;

    @Autowired
    private ExamineInfoService examineInfoService;
    //页面
    @RequestMapping(value = "warrantyListPage")
    public String warrantyListPage(Model model) {
        User user = UserUtils.getUser();
        model.addAttribute("userId",user.getId());
        model.addAttribute("formId", FormUtils.WITHDRAWAL_WARRANTY_APPLICATION);
        return "modules/flowpath/warrantyListPage";
    }

    @ResponseBody
    @RequestMapping(value = "warrantyListData")
    public String warrantyListData(HttpServletRequest request){
        Map<String, String> map = getParams(request);
        LayuiPage page = new LayuiPage(request);
        warrantyApplyService.applyList(page,map);
        return page.renderJson();
    }

    //新增页面
    @RequestMapping(value = "addWarrantyDepositPage")
    public String addDistributionGoodsPage(Model model){
        List<Record> list=distributionGoodsApplyService.getTrader();
        model.addAttribute("traders",list);
        model.addAttribute("imgsCount", 0);
        return "modules/flowpath/addWarrantyDepositPage";
    }

    //新建/修改申请
    @ResponseBody
    @RequestMapping(value = "saveWarrantyDeposit")
    public Result saveWarrantyDeposit(@RequestBody DistributionGoodsVo vo){
        try{
            warrantyApplyService.saveWarrantyDeposit(vo);
            return Result.ok();
        }catch (RuntimeException r){
            r.printStackTrace();
            return Result.fail(r.getMessage());
        }catch (Exception e){
            e.printStackTrace();
            logger.error(">>>>>>>>>>>saveWarrantyDeposit e:{}",e);
        }
        return Result.fail("保存失败");
    }

    //编辑页面
    @RequestMapping(value = "editWarrantyDepositPage")
    public String editWarrantyDepositPage(HttpServletRequest request,Model model){
        String id=request.getParameter("id");
        Record apply=warrantyApplyService.getDetailById(id);
        model.addAttribute("apply",apply);
        List<Record> list=distributionGoodsApplyService.getTrader();
        model.addAttribute("traders",list);
       /* List<ExamineInfo> infos=examineInfoService.getByApplyId(id);
        model.addAttribute("infos",infos);*/
        int imgsCount = 0;
        if (StringUtils.isNotBlank(apply.getStr("voucher"))) {
            String[] imgArr = apply.getStr("voucher").split(",");
            imgsCount = imgArr.length;
            List<String> imglist = Arrays.asList(imgArr);
            model.addAttribute("imglist", imglist);
        }
        model.addAttribute("imgsCount", imgsCount);
        return "modules/flowpath/addWarrantyDepositPage";
    }

    //查看详情
    @RequestMapping(value = "applyWarrantyDeposit")
    public String applyWarrantyDeposit(HttpServletRequest request,Model model){
        String id=request.getParameter("id");
        String from=request.getParameter("from"); //from = audit 那么就是审批操作  否则就是查看
        model.addAttribute("from",from);
        Record apply=warrantyApplyService.getDetailById(id);
        model.addAttribute("apply",apply);
        List<ExamineInfo> infos=examineInfoService.getByApplyId(id,FormUtils.WITHDRAWAL_WARRANTY_APPLICATION);
        model.addAttribute("infos",infos);
        if (StringUtils.isNotBlank(apply.getStr("voucher"))) {
            String[] imgArr = apply.getStr("voucher").split(",");
            List<String> imglist = Arrays.asList(imgArr);
            model.addAttribute("imglist", imglist);
        }
        User user = UserUtils.getUser();
        model.addAttribute("userType",user.getUserType());
        if(!"audit".equals(from)){
             warrantyApplyService.readApplication(apply.getStr("process_instance_id"),FormUtils.WITHDRAWAL_WARRANTY_APPLICATION,id,user);
        }
        return "modules/flowpath/applyWarrantyDepositDetail";
    }

    //删除申请
    @ResponseBody
    @RequestMapping(value = "deleteWarrantyDepositApply")
    public Result deleteWarrantyDepositApply(HttpServletRequest request){
        String id=request.getParameter("id");
        try{
            warrantyApplyService.deleteWarrantyDepositApply(id);
            return Result.ok();
        }catch (RuntimeException r){
            r.printStackTrace();
            return Result.fail(r.getMessage());
        }catch (Exception e){
            logger.error(">>>>>>>>>>>>>>deleteWarrantyDepositApply e:{}",e);
            e.printStackTrace();
            return Result.fail("删除失败！");
        }
    }

    //审核通过
    @ResponseBody
    @RequestMapping(value = "examineWarrantyDepositApprove")
    public Result examineWarrantyDepositApprove(UserApplyVo userApplyVo){
        try{
            warrantyApplyService.examineWarrantyDepositApprove(userApplyVo.getId(),userApplyVo.getTaskId(),userApplyVo.getContent());
            return Result.ok();
        }catch (Exception e){
            logger.error(">>>>>>>>>>>>>>examineWarrantyDepositApprove e:{}",e);
            return Result.fail("审核失败！");
        }
    }

    //审核驳回
    @ResponseBody
    @RequestMapping(value = "rejectWarrantyDepositTask")
    public Result rejectWarrantyDepositTask(@Valid UserApplyVo userApplyVo){
        try{
            warrantyApplyService.rejectWarrantyDepositTask(userApplyVo.getId(),userApplyVo.getTaskId(),userApplyVo.getContent());
            return Result.ok();
        }catch (Exception e){
            logger.error(">>>>>>>>>>>>>>rejectWarrantyDepositTask e:{}",e);
            return Result.fail("审核失败！");
        }
    }



}
