package com.sifang.modules.insurance.handler;

import com.google.common.collect.Maps;
import com.sifang.modules.insurance.entity.CrmCustomerEntryInsurance;
import com.sifang.modules.order.utils.excelExt.handler.ExcelAbstractHandler;
import com.sifang.modules.sys.common.utils.DateUtils;
import org.apache.commons.lang.StringUtils;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class InsuranceExcelImportHandler  extends ExcelAbstractHandler {

    private List<CrmCustomerEntryInsurance> retList = new ArrayList<>();

    private List<String> insuranceCodeList = new ArrayList<>();

    boolean checkFlag;

    private int maxRows = 5000;   //导入的最大条数


    @Override
    public Boolean handler(Map<String, Object> rowData, int rowIndex) {
        CrmCustomerEntryInsurance addTrader = transToModel(rowData);
        retList.add(addTrader);
        return true;
    }

    private CrmCustomerEntryInsurance transToModel(Map<String, Object> rowData){
        CrmCustomerEntryInsurance insurance = new CrmCustomerEntryInsurance();
        insurance.setItemNo(formatCode(rowData.get("1")));
        insurance.setNumber(formatCode(rowData.get("2")));
        insurance.setInsuranceMoney(new BigDecimal(rowData.get("3").toString()));
        insurance.setPrice(new BigDecimal(rowData.get("4").toString()));
        insurance.setInsuranceCompanyName(formatCode(rowData.get("5")));
        insurance.setPolicyholder(formatCode(rowData.get("6")));
        insurance.setInsurant(formatCode(rowData.get("7")));
        try {
            insurance.setEndTime(DateUtils.convertExcelDateDayToJavaDate(Double.parseDouble(String.valueOf(rowData.get("9")))));
            insurance.setStartTime(DateUtils.convertExcelDateDayToJavaDate(Double.parseDouble(String.valueOf(rowData.get("8")))));
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        return insurance;
    }

    //匹配模板
    private boolean matchTemplate(Map<String, Object> rowData){
        List<String> titles = new ArrayList<>();
        titles.add("电站编码");
        titles.add("保单编号");
        titles.add("投保金额");
        titles.add("保费");
        titles.add("投保公司");
        titles.add("投保人");
        titles.add("被保人");
        titles.add("保险开始时间");
        titles.add("保险结束时间");
        return checkTemplate(titles, rowData);
    }

    public boolean checkTemplate(List<String> titles, Map<String, Object> rowData){
        boolean ret = true;
        for (int i = 0; i < titles.size(); i++) {
            String title = String.valueOf(rowData.get(String.valueOf(i + 1)));
            if (!titles.get(i).equals(title.trim())) {
                //模板不匹配
                ret = false;
                break;
            }
        }
        return ret;
    }

    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    //验证Excel模板
    @SuppressWarnings("unchecked")
    @Override
    public Boolean checkExcel(Map<String, Object> rowData, int rowIndex) {
        Map<String, Object> result = (Map<String, Object>) this.getHandlerResult();
        if (result == null) {
            result = Maps.newHashMap();
            this.setHandlerResult(result);
        }
        //如果是title列，则直接检查是否同一个Excel模板,包括顺序和列名匹配
        if (rowIndex == (getStartRow() - 1)) {
            boolean match = matchTemplate(rowData);
            boolean flag = true;
            if(!match){
                result.put("TemplateError", "<p>模板不匹配</p>");
                flag = false;
            }
            return flag;
        }

        if (rowIndex > (getStartRow() - 1)) {
            checkFlag = true;
            if (rowData.get("1") == null || StringUtils.isBlank(String.valueOf(rowData.get("1")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：电站编码为空" + "</p>");
                return true;
            }

            if (rowData.get("2") == null || StringUtils.isBlank(String.valueOf(rowData.get("2")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：保单编号为空" + "</p>");
                return true;
            }

            if (rowData.get("3") == null || StringUtils.isBlank(String.valueOf(rowData.get("3")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：投保金额为空" + "</p>");
                return true;
            }

            if (rowData.get("4") == null || StringUtils.isBlank(String.valueOf(rowData.get("4")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：保费为空" + "</p>");
                return true;
            }

            if (rowData.get("5") == null || StringUtils.isBlank(String.valueOf(rowData.get("5")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：投保公司为空" + "</p>");
                return true;
            }

            if (rowData.get("6") == null || StringUtils.isBlank(String.valueOf(rowData.get("6")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：投保人为空" + "</p>");
                return true;
            }

            if (rowData.get("7") == null || StringUtils.isBlank(String.valueOf(rowData.get("7")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：被保人为空" + "</p>");
                return true;
            }

            if (rowData.get("8") == null || StringUtils.isBlank(String.valueOf(rowData.get("8")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：保险开始时间为空" + "</p>");
                return true;
            }

            if (rowData.get("9") == null || StringUtils.isBlank(String.valueOf(rowData.get("9")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：保险结束时间为空" + "</p>");
                return true;
            }

            retList.add(transToModel(rowData));
            if (rowIndex > maxRows) {
                result.put("overLimit", "overLimit");
                return false;
            }
        }

        return true;
    }

    private String formatCode(Object code) {
        if (code == null) {
            return "";
        }
        Object codeVal = code;
        if (codeVal instanceof Double) {
            codeVal = new DecimalFormat("#").format((Double) codeVal);
        } else {
            codeVal = code;
        }
        return String.valueOf(codeVal);
    }

    public List<CrmCustomerEntryInsurance> getRetList() {
        return retList;
    }

    public void setRetList(List<CrmCustomerEntryInsurance> retList) {
        this.retList = retList;
    }

    public boolean isCheckFlag() {
        return checkFlag;
    }

    public void setCheckFlag(boolean checkFlag) {
        this.checkFlag = checkFlag;
    }
}
