package com.sifang.modules.sunshine.SystemSetting.handler;

import com.google.common.collect.Maps;
import com.sifang.modules.order.utils.excelExt.handler.ExcelAbstractHandler;
import com.sifang.modules.sunshine.SystemSetting.form.AddAuthorizedArea;
import org.apache.commons.lang.StringUtils;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ExcelImportAuthorizedAreaNumberHandler extends ExcelAbstractHandler {

    private List<AddAuthorizedArea> additionalAreaList =  new ArrayList<>();

    boolean checkFlag;
    private int maxRows=5000;   //导入的最大条数

    @Override
    public Boolean handler(Map<String, Object> rowData, int rowIndex) {
        AddAuthorizedArea addAuthorizedArea = transToModel(rowData);
        additionalAreaList.add(addAuthorizedArea);
        return true;
    }

    private AddAuthorizedArea transToModel(Map<String, Object> rowData) {
        AddAuthorizedArea addAuthorizedArea = new AddAuthorizedArea();
        addAuthorizedArea.setName(formatCode(rowData.get("2")));
        addAuthorizedArea.setProvince(formatCode(rowData.get("3")));
        addAuthorizedArea.setCity(formatCode(rowData.get("4")));
        addAuthorizedArea.setArea(formatCode(rowData.get("5")));
        return addAuthorizedArea;
    }

    private String formatCode(Object code) {
        if (code == null) {
            return "";
        }
        Object codeVal = code;
        if (codeVal instanceof Double) {
            codeVal = new DecimalFormat("#").format((Double) codeVal);
        } else {
            codeVal = code;
        }
        return String.valueOf(codeVal);
    }
    @Override
    public Boolean checkExcel(Map<String, Object> rowData, int rowIndex) {
        Map<String, Object> result = (Map<String, Object>) this.getHandlerResult();
        if (result == null) {
            result = Maps.newHashMap();
            this.setHandlerResult(result);
        }
        // check excel title match
        if (rowIndex == (getStartRow() - 1)) {//如果是title列，则直接检查是否同一个Excel模板,包括顺序和列名匹配
            boolean match = matchTemplate(rowData);
            boolean flag = true;
            if(!match){
                result.put("TemplateError", "<p>模板不匹配</p>");
                flag = false;
            }
            return flag;
        }
        if (rowIndex > (getStartRow() -1)) {
            checkFlag = true;
            if (rowData.get("2") == null || StringUtils.isBlank(String.valueOf(rowData.get("2")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：公司名称为空" + "</p>");
            }
            if (rowData.get("3") == null || StringUtils.isBlank(rowData.get("3").toString())) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：省为空！" + "</p>");
            }
            if(rowData.get("5") != null && StringUtils.isNotBlank(rowData.get("5").toString())&&
                    ( rowData.get("4") == null || StringUtils.isBlank(rowData.get("4").toString()))){
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：市为空！" + "</p>");
            }
            additionalAreaList.add(transToModel(rowData));
            if (rowIndex > maxRows) {
                result.put("overLimit", "overLimit");
                return false;
            }
        }
        return true;
    }

    private boolean matchTemplate(Map<String, Object> rowData) {
        List<String> titles = new ArrayList<>();
        titles.add("序号");
        titles.add("公司名称");
        titles.add("省");
        titles.add("市");
        titles.add("区");
        return checkTemplate(titles, rowData);
    }

    private boolean checkTemplate(List<String> titles, Map<String, Object> rowData) {
        boolean ret = true;
        for (int i = 0; i < titles.size(); i++) {
            String title = String.valueOf(rowData.get(String.valueOf(i + 1)));
            if (!titles.get(i).equals(title.trim())) {
                //模板不匹配
                ret = false;
                break;
            }
        }
        return ret;
    }

    public List<AddAuthorizedArea> getAdditionalAreaList() {
        return additionalAreaList;
    }

    public void setAdditionalAreaList(List<AddAuthorizedArea> additionalAreaList) {
        this.additionalAreaList = additionalAreaList;
    }

    public boolean isCheckFlag() {
        return checkFlag;
    }

    public void setCheckFlag(boolean checkFlag) {
        this.checkFlag = checkFlag;
    }

    @Override
    public int getMaxRows() {
        return maxRows;
    }

    @Override
    public void setMaxRows(int maxRows) {
        this.maxRows = maxRows;
    }
}
