package com.sifang.modules.sunshine.handler;

import com.google.common.collect.Maps;
import com.jfinal.plugin.activerecord.Record;
import com.sifang.modules.order.utils.excelExt.handler.ExcelAbstractHandler;
import com.sifang.modules.sunshine.SystemSetting.entity.CrmProjectContract;
import com.sifang.modules.sunshine.SystemSetting.service.ProjectContractService;
import com.sifang.modules.sys.util.SpringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ContractExcelImportHandler extends ExcelAbstractHandler {
    private List<Map<String,Object>> errorRowData=new ArrayList<>();
    public List<Map<String, Object>> getErrorRowData() {
        return errorRowData;
    }

    private List<CrmProjectContract> retList = new ArrayList<>();

    private List<String> materialCodeList = new ArrayList<>();

    boolean checkFlag;

    private int maxRows=5000;   //导入的最大条数


    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    @SuppressWarnings("unchecked")
    @Override
    public Boolean handler(Map<String, Object> rowData, int rowIndex) {
        CrmProjectContract addTrader = transToModel(rowData);
        retList.add(addTrader);
        return true;
    }

    private CrmProjectContract transToModel(Map<String, Object> rowData){
        CrmProjectContract material = new CrmProjectContract();
        material.setCustomerName(String.valueOf(rowData.get("1")));
        material.setIdCode(String.valueOf(rowData.get("2")));
        material.setAssemblySpecs(new BigDecimal(rowData.get("3").toString()).setScale(2, RoundingMode.HALF_UP));
        String num = String.valueOf(rowData.get("4"));
        Double dNum = Double.parseDouble(num);
        material.setAssemblyNum(dNum.intValue());
        return material;
    }

    //匹配模板
    private boolean matchTemplate(Map<String, Object> rowData){
        List<String> titles = new ArrayList<>();
        titles.add("业主姓名");
        titles.add("身份证号");
        titles.add("安装组件规格");
        titles.add("安装组件块数");
        return checkTemplate(titles, rowData);
    }

    public boolean checkTemplate(List<String> titles, Map<String, Object> rowData){
        boolean ret = true;
        for (int i = 0; i < titles.size(); i++) {
            String title = String.valueOf(rowData.get(String.valueOf(i + 1)));
            if (!titles.get(i).equals(title.trim())) {
                //模板不匹配
                ret = false;
                break;
            }
        }
        return ret;
    }

    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    //验证Excel模板
    @SuppressWarnings("unchecked")
    @Override
    public Boolean checkExcel(Map<String, Object> rowData, int rowIndex) {
        Map<String, Object> result = (Map<String, Object>) this.getHandlerResult();
        if (result == null) {
            result = Maps.newHashMap();
            this.setHandlerResult(result);
        }
        // check excel title match
        if (rowIndex == (getStartRow() - 1)) {//如果是title列，则直接检查是否同一个Excel模板,包括顺序和列名匹配
            boolean match = matchTemplate(rowData);
            boolean flag = true;
            if(!match){
                result.put("TemplateError", "<p>模板不匹配</p>");
                flag = false;
            }
            return flag;
        }

        if (rowIndex > (getStartRow() - 1)) {
            checkFlag = true;
            if (rowData.get("1") == null || StringUtils.isBlank(String.valueOf(rowData.get("1")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：业主姓名为空" + "</p>");
                return true;
            }

            if (rowData.get("2") == null || StringUtils.isBlank(String.valueOf(rowData.get("2")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：身份证号为空" + "</p>");
                return true;
            }

            if (rowData.get("3") == null || StringUtils.isBlank(String.valueOf(rowData.get("3")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：安装组件规格为空" + "</p>");
                return true;
            }
            if (rowData.get("4") == null || StringUtils.isBlank(String.valueOf(rowData.get("4")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：安装组件块数为空" + "</p>");
                return true;
            }
            try {
               double num = Double.parseDouble(String.valueOf(rowData.get("4")));
            }catch (Exception e){
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：安装组件块数不正确" + "</p>");
                return true;
            }
            if (materialCodeList.contains(String.valueOf(rowData.get("1"))+String.valueOf(rowData.get("2"))+String.valueOf(rowData.get("3"))+String.valueOf(rowData.get("4")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：业主姓名+身份证号+安装组件规格+安装组件块数 重复" + "</p>");
                return true;
            }
            materialCodeList.add(String.valueOf(rowData.get("1"))+String.valueOf(rowData.get("2"))+String.valueOf(rowData.get("3"))+String.valueOf(rowData.get("4")));


            retList.add(transToModel(rowData));
            if (isLastRow(rowIndex)) {
                checkMaterialCodeExists(retList);
            }
            if (rowIndex > maxRows) {
                result.put("overLimit", "overLimit");
                return false;
            }
        }

        return true;
    }

    private void checkMaterialCodeExists(List<CrmProjectContract> materials){
        ProjectContractService planService = SpringUtils.getBean(ProjectContractService.class);
        if (CollectionUtils.isEmpty(materials)) {
            return;
        }
        List<String> codeList = new ArrayList<>();
        for (CrmProjectContract con : materials) {
            codeList.add(con.getCustomerName().trim()+con.getIdCode().trim()+con.getAssemblySpecs()+con.getAssemblyNum());
        }
        List<String> existsCode = planService.findExistsCode("");
        for (int i = 0; i < codeList.size(); i++) {
            if (existsCode.contains(codeList.get(i))) {
                appendErrorDetail("第" + (i + 1) + "行信息已存在</p>");
            }
        }
    }

    private List<String> getNames(List<Record> records, String column){
        List<String> names = new ArrayList<>();
        for(Record record: records){
            names.add(record.getStr(column));
        }
        return names;
    }

    private boolean isLastRow(int rowIndex) {
        return rowIndex >= totalRow - 1;
    }

    public List<CrmProjectContract> getRetList() {
        return retList;
    }

    public void setRetList(List<CrmProjectContract> retList) {
        this.retList = retList;
    }

    public boolean isCheckFlag() {
        return checkFlag;
    }

    public void setCheckFlag(boolean checkFlag) {
        this.checkFlag = checkFlag;
    }
}
