package com.sifang.modules.sunshine.handler;

import com.google.common.collect.Maps;
import com.jfinal.plugin.activerecord.Record;
import com.sifang.modules.order.utils.excelExt.handler.ExcelAbstractHandler;
import com.sifang.modules.sunshine.SystemSetting.service.ProjectContractService;
import com.sifang.modules.sys.util.SpringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class CustomerEntryRentHandler extends ExcelAbstractHandler {
    private List<Map<String,Object>> errorRowData=new ArrayList<>();
    public List<Map<String, Object>> getErrorRowData() {
        return errorRowData;
    }

    private List<Map<String, Object>> retList = new ArrayList<>();


    boolean checkFlag;

    private int maxRows=5000;   //导入的最大条数


    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    @SuppressWarnings("unchecked")
    @Override
    public Boolean handler(Map<String, Object> rowData, int rowIndex) {
        if("是".equals(String.valueOf(rowData.get("14")))){
            Map<String,Object> map = Maps.newHashMap();
            map.put("customerEntryCode",String.valueOf(rowData.get("1")).trim());
            map.put("payTime",rowData.get("13"));
            retList.add(map);
        }
        return true;
    }

    //匹配模板
    private boolean matchTemplate(Map<String, Object> rowData){
        List<String> titles = new ArrayList<>();
        titles.add("电站编码");
        titles.add("产品编号");
        titles.add("产品名称");
        titles.add("代理商名称");
        titles.add("代理商电话");
        titles.add("租户姓名");
        titles.add("联系电话");
        titles.add("上期租金");
        titles.add("支付状态");
        titles.add("当期状态");
        titles.add("支付日期");
        titles.add("当期应付");
        titles.add("应付日期");
        titles.add("是否支付");
        return checkTemplate(titles, rowData);
    }

    public boolean checkTemplate(List<String> titles, Map<String, Object> rowData){
        boolean ret = true;
        for (int i = 0; i < titles.size(); i++) {
            String title = String.valueOf(rowData.get(String.valueOf(i + 1)));
            if (!titles.get(i).equals(title.trim())) {
                //模板不匹配
                ret = false;
                break;
            }
        }
        return ret;
    }

    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    //验证Excel模板
    @SuppressWarnings("unchecked")
    @Override
    public Boolean checkExcel(Map<String, Object> rowData, int rowIndex) {
        Map<String, Object> result = (Map<String, Object>) this.getHandlerResult();
        if (result == null) {
            result = Maps.newHashMap();
            this.setHandlerResult(result);
        }
        // check excel title match
        if (rowIndex == (getStartRow() - 1)) {//如果是title列，则直接检查是否同一个Excel模板,包括顺序和列名匹配
            boolean match = matchTemplate(rowData);
            boolean flag = true;
            if(!match){
                result.put("TemplateError", "<p>模板不匹配</p>");
                flag = false;
            }
            return flag;
        }

        if (rowIndex > (getStartRow() - 1)) {
            checkFlag = true;
            if (rowData.get("1") == null || StringUtils.isBlank(String.valueOf(rowData.get("1")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：电站编码为空" + "</p>");
                return true;
            }

       /*     if(retList.contains(rowData.get("1"))){
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：电站编码重复" + "</p>");
                return true;
            }*/
            if(rowData.get("14") != null || StringUtils.isNotBlank(String.valueOf(rowData.get("14")))){
                if("是".equals(String.valueOf(rowData.get("14")))){
                    Map<String,Object> map = Maps.newHashMap();
                    map.put("customerEntryCode",String.valueOf(rowData.get("1")).trim());
                    map.put("payTime",rowData.get("13"));
                    retList.add(map);
                }
            }

          /*  if (isLastRow(rowIndex)) {
                checkMaterialCodeExists(retList);
            }*/
            if (rowIndex > maxRows) {
                result.put("overLimit", "overLimit");
                return false;
            }
        }

        return true;
    }

    private void checkMaterialCodeExists(List<String> materials){
        ProjectContractService planService = SpringUtils.getBean(ProjectContractService.class);
        if (CollectionUtils.isEmpty(materials)) {
            return;
        }

        List<String> existsCode = planService.findExistsCode("");
        for (int i = 0; i < materials.size(); i++) {
            if (existsCode.contains(materials.get(i))) {
                appendErrorDetail("第" + (i + 1) + "行电站编码信息状态不满足</p>");
            }
        }
    }

    private List<String> getNames(List<Record> records, String column){
        List<String> names = new ArrayList<>();
        for(Record record: records){
            names.add(record.getStr(column));
        }
        return names;
    }

    private boolean isLastRow(int rowIndex) {
        return rowIndex >= totalRow - 1;
    }

    public List<Map<String,Object>> getRetList() {
        return retList;
    }

    public void setRetList(List<Map<String,Object>> retList) {
        this.retList = retList;
    }

    public boolean isCheckFlag() {
        return checkFlag;
    }

    public void setCheckFlag(boolean checkFlag) {
        this.checkFlag = checkFlag;
    }
}
