package com.sifang.modules.sunshine.handler;

import com.google.common.collect.Maps;
import com.jfinal.plugin.activerecord.Db;
import com.jfinal.plugin.activerecord.Record;
import com.sifang.modules.order.utils.excelExt.handler.ExcelAbstractHandler;
import com.sifang.modules.sunshine.SystemSetting.form.AddElectricMaterialSet;
import com.sifang.modules.sunshine.SystemSetting.form.AddPlanMaterial;
import com.sifang.modules.sunshine.SystemSetting.service.PlanService;
import com.sifang.modules.sys.util.SpringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ElectricMaterialImportHandler extends ExcelAbstractHandler {

    private List<Map<String,Object>> errorRowData=new ArrayList<>();
    public List<Map<String, Object>> getErrorRowData() {
        return errorRowData;
    }

    private List<AddElectricMaterialSet> retList = new ArrayList<>();

    private List<String> materialCodeList = new ArrayList<>();

    private List<String> materialModelList=new ArrayList<>();

    boolean checkFlag;


    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    @SuppressWarnings("unchecked")
    @Override
    public Boolean handler(Map<String, Object> rowData, int rowIndex) {
        AddElectricMaterialSet set = transToModel(rowData);
        retList.add(set);
        return true;
    }

    private AddElectricMaterialSet transToModel(Map<String, Object> rowData){
        AddElectricMaterialSet set = new AddElectricMaterialSet();
        set.setCode(String.valueOf(rowData.get("1")));
        String type=String.valueOf(rowData.get("2")).trim();
        if("逆变器".equals(type)){
            set.setType("1");
        }else if("并网箱".equals(type)){
            set.setType("2");
        }
        set.setName(String.valueOf(rowData.get("3")));
        set.setModel(String.valueOf(rowData.get("4")));
        set.setUnit(String.valueOf(rowData.get("5")));
        return set;
    }

    //匹配模板
    private boolean matchTemplate(Map<String, Object> rowData){
        List<String> titles = new ArrayList<>();
        titles.add("料号");
        titles.add("类型");
        titles.add("名称");
        titles.add("规格");
        titles.add("单位");
        return checkTemplate(titles, rowData);
    }

    public boolean checkTemplate(List<String> titles, Map<String, Object> rowData){
        boolean ret = true;
        for (int i = 0; i < titles.size(); i++) {
            String title = String.valueOf(rowData.get(String.valueOf(i + 1)));
            if (!titles.get(i).equals(title.trim())) {
                //模板不匹配
                ret = false;
                break;
            }
        }
        return ret;
    }

    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    //验证Excel模板
    @SuppressWarnings("unchecked")
    @Override
    public Boolean checkExcel(Map<String, Object> rowData, int rowIndex) {
        Map<String, Object> result = (Map<String, Object>) this.getHandlerResult();
        if (result == null) {
            result = Maps.newHashMap();
            this.setHandlerResult(result);
        }
        if (rowIndex == (getStartRow() - 1)) {//如果是title列，则直接检查是否同一个Excel模板,包括顺序和列名匹配
            boolean match = matchTemplate(rowData);
            boolean flag = true;
            if(!match){
                result.put("TemplateError", "<p>模板不匹配</p>");
                flag = false;
            }
            return flag;
        }

        if (rowIndex > (getStartRow() - 1)) {
            checkFlag = true;
            if (rowData.get("1") == null || StringUtils.isBlank(String.valueOf(rowData.get("1")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：料号为空" + "</p>");
                return true;
            }
            if (materialCodeList.contains(String.valueOf(rowData.get("1")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：料号重复" + "</p>");
                return true;
            }
            materialCodeList.add(String.valueOf(rowData.get("1")));

            if (rowData.get("2") == null || StringUtils.isBlank(String.valueOf(rowData.get("2")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：类型为空" + "</p>");
                return true;
            }
            String type=String.valueOf(rowData.get("2")).trim();
            if(!"逆变器".equals(type) && !"并网箱".equals(type)){
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：类型不存在 " + "</p>");
                return true;
            }
            if (rowData.get("3") == null || StringUtils.isBlank(String.valueOf(rowData.get("3")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：名称为空" + "</p>");
                return true;
            }
            if (rowData.get("4") == null || StringUtils.isBlank(String.valueOf(rowData.get("4")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：规格为空" + "</p>");
                return true;
            }
            if (rowData.get("5") == null || StringUtils.isBlank(String.valueOf(rowData.get("5")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：单位为空" + "</p>");
                return true;
            }

            retList.add(transToModel(rowData));
            if (isLastRow(rowIndex)) {
                checkMaterialCodeModelExists(retList);
            }
            if (rowIndex > getMaxRows()) {
                result.put("overLimit", "overLimit");
                return false;
            }
            if (rowIndex > getMaxRows()) {
                result.put("overLimit", "overLimit");
                return false;
            }
        }

        return true;
    }

    private void checkMaterialCodeModelExists(List<AddElectricMaterialSet> sets){
        PlanService planService = SpringUtils.getBean(PlanService.class);
        if (CollectionUtils.isEmpty(sets)) {
            return;
        }
        List<String> codeList = new ArrayList<>();
        List<String> modelList=new ArrayList<>();
        for (AddElectricMaterialSet set : sets) {
            codeList.add(set.getCode());
            modelList.add(set.getModel());
        }
        List<String> existsCode = planService.findCommonCode(codeList);
        List<String> existsModel=planService.findCommonModel(modelList);
        for (int i = 0; i < codeList.size(); i++) {
            if (existsCode.contains(codeList.get(i))) {
                appendErrorDetail("第" + (i + 1) + "行料号已存在");
            }
        }
        for (int i = 0; i < modelList.size(); i++) {
            if (existsModel.contains(modelList.get(i))) {
                appendErrorDetail("第" + (i + 1) + "行规格已存在");
            }
        }
    }



    private boolean isLastRow(int rowIndex) {
        return rowIndex >= totalRow - 1;
    }

    public List<AddElectricMaterialSet> getRetList() {
        return retList;
    }

    public boolean isCheckFlag() {
        return checkFlag;
    }

    public void setCheckFlag(boolean checkFlag) {
        this.checkFlag = checkFlag;
    }
}
