package com.sifang.modules.supply.stock.handler;

import com.google.common.collect.Maps;
import com.jfinal.plugin.activerecord.Record;
import com.sifang.modules.order.utils.excelExt.handler.ExcelAbstractHandler;
import com.sifang.modules.sunshine.SystemSetting.form.AddPlanMaterial;
import com.sifang.modules.sunshine.SystemSetting.service.PlanService;
import com.sifang.modules.supply.stock.form.AddMaterial;
import com.sifang.modules.sys.util.SpringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.util.CollectionUtils;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ExcelImportMaterialHandler extends ExcelAbstractHandler {
    private List<Map<String,Object>> errorRowData=new ArrayList<>();
    public List<Map<String, Object>> getErrorRowData() {
        return errorRowData;
    }

    private List<AddMaterial> retList = new ArrayList<>();

    private List<String> materialCodeList = new ArrayList<>();

    boolean checkFlag;

    private int maxRows=5000;   //导入的最大条数
    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    @SuppressWarnings("unchecked")
    @Override
    public Boolean handler(Map<String, Object> rowData, int rowIndex) {
        AddMaterial addTrader = transToModel(rowData);
        retList.add(addTrader);
        return true;
    }

    private AddMaterial transToModel(Map<String, Object> rowData){
        AddMaterial material = new AddMaterial();
        material.setCode(formatCode(rowData.get("1")));
        material.setQqCode(formatCode(rowData.get("2")));
        material.setMaterialName(String.valueOf(rowData.get("3")));
        material.setMaterialType(String.valueOf(rowData.get("4")));
        material.setMaterialSpecs(formatCode(rowData.get("5")));
        material.setMaterialModel(String.valueOf(rowData.get("6")));
        material.setMaterialBrand(String.valueOf(rowData.get("7")));
        material.setUnitName(String.valueOf(rowData.get("8")));
//        if(rowData.get("8")==null || "".equals(rowData.get("8"))){
//            material.setUnit(String.valueOf(rowData.get("7")));
//        }else{
        //material.setUnit(String.valueOf(rowData.get("9")));
        //       }
        material.setComponentShape(String.valueOf(rowData.get("9")));
        material.setBatteryParam(String.valueOf(rowData.get("10")));
        String remarks = formatCode(rowData.get("11"));
        material.setRemarks(remarks);
        return material;
    }

    //匹配模板
    private boolean matchTemplate(Map<String, Object> rowData){
        List<String> titles = new ArrayList<>();
        titles.add("物料代码");
        titles.add("外部编码");
        titles.add("物料名称");
        titles.add("物料类型");
        titles.add("物料规格");
        titles.add("物料型号");
        titles.add("物料品牌");
        titles.add("库存单位");
        titles.add("组件玻型");
        titles.add("电池参数");
        titles.add("物料描述");
        return checkTemplate(titles, rowData);
    }

    public boolean checkTemplate(List<String> titles, Map<String, Object> rowData){
        boolean ret = true;
        for (int i = 0; i < titles.size(); i++) {
            String title = String.valueOf(rowData.get(String.valueOf(i + 1)));
            if (!titles.get(i).equals(title.trim())) {
                //模板不匹配
                ret = false;
                break;
            }
        }
        return ret;
    }

    /**
     * 如果返回false，则表示终止继续遍历Excel,截至到当前的行数
     */
    //验证Excel模板
    @SuppressWarnings("unchecked")
    @Override
    public Boolean checkExcel(Map<String, Object> rowData, int rowIndex) {
        Map<String, Object> result = (Map<String, Object>) this.getHandlerResult();
        if (result == null) {
            result = Maps.newHashMap();
            this.setHandlerResult(result);
        }
        // check excel title match
        if (rowIndex == (getStartRow() - 1)) {//如果是title列，则直接检查是否同一个Excel模板,包括顺序和列名匹配
            boolean match = matchTemplate(rowData);
            boolean flag = true;
            if(!match){
                result.put("TemplateError", "<p>模板不匹配</p>");
                flag = false;
            }
            return flag;
        }

        if (rowIndex > (getStartRow() - 1)) {
            checkFlag = true;
            if (rowData.get("1") == null || StringUtils.isBlank(String.valueOf(rowData.get("1")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料代码为空" + "</p>");
            }
            if (materialCodeList.contains(String.valueOf(rowData.get("1")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料代码重复" + "</p>");
            }
            materialCodeList.add(String.valueOf(rowData.get("1")));
            if (rowData.get("2") == null || StringUtils.isBlank(String.valueOf(rowData.get("2")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：外部编码为空" + "</p>");
            }
            if (rowData.get("3") == null || StringUtils.isBlank(String.valueOf(rowData.get("3")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料名称为空" + "</p>");
            }
            if (rowData.get("4") == null || StringUtils.isBlank(String.valueOf(rowData.get("4")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料类型为空" + "</p>");
            } else if ("组件".equals(rowData.get("4")) && !formatCode(rowData.get("5")).matches("^\\d+(\\.\\d{1,2})?$")) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料类型为组件时，物料规格必须是纯数字！" + "</p>");
                return true;
            }
            if (rowData.get("5") == null || StringUtils.isBlank(String.valueOf(rowData.get("5")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料规格为空" + "</p>");
            }
            if (rowData.get("6") == null || StringUtils.isBlank(String.valueOf(rowData.get("6")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料型号为空" + "</p>");
            }
            if (rowData.get("7") == null || StringUtils.isBlank(String.valueOf(rowData.get("7")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：物料品牌为空" + "</p>");
            }
            if (rowData.get("8") == null || StringUtils.isBlank(String.valueOf(rowData.get("8")))) {
                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：库存单位为空" + "</p>");
            }
//            if (rowData.get("9") == null || StringUtils.isBlank(String.valueOf(rowData.get("9")))) {
//                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：零售价格为空" + "</p>");
//            } else
//            if (!formatCode(rowData.get("9")).matches("(^[1-9]([0-9]+)?(\\.[0-9]{1,2})?$)|(^(0){1}$)|(^[0-9]\\.[0-9]([0-9])?$)")) {
//                appendErrorDetail("<p>第" + (rowIndex + 1) + "行数据：零售价格格式不正确！" + "</p>");
//                return true;
//            }

            retList.add(transToModel(rowData));
           /* if (isLastRow(rowIndex)) {
                checkMaterialCodeExists(retList);
            }*/
            if (rowIndex > maxRows) {
                result.put("overLimit", "overLimit");
                return false;
            }
        }

        return true;
    }

    private void checkMaterialCodeExists(List<AddMaterial> materials){
        PlanService planService = SpringUtils.getBean(PlanService.class);
        if (CollectionUtils.isEmpty(materials)) {
            return;
        }
        List<String> codeList = new ArrayList<>();
        for (AddMaterial material : materials) {
            codeList.add(material.getCode());
        }
        List<String> existsCode = planService.findExistsCode(codeList, "");
        for (int i = 0; i < codeList.size(); i++) {
            if (existsCode.contains(codeList.get(i))) {
                appendErrorDetail("第" + (i + 1) + "行料号已存在");
            }
        }
    }

    private List<String> getNames(List<Record> records, String column){
        List<String> names = new ArrayList<>();
        for(Record record: records){
            names.add(record.getStr(column));
        }
        return names;
    }

    private String formatCode(Object code) {
        if (code == null) {
            return "";
        }
        Object codeVal = code;
        if (codeVal instanceof Double) {
            codeVal = new DecimalFormat("#").format((Double) codeVal);
        } else {
            codeVal = code;
        }
        return String.valueOf(codeVal);
    }

    private boolean isLastRow(int rowIndex) {
        return rowIndex >= totalRow - 1;
    }

    public List<AddMaterial> getRetList() {
        return retList;
    }

    public void setRetList(List<AddMaterial> retList) {
        this.retList = retList;
    }

    public boolean isCheckFlag() {
        return checkFlag;
    }

    public void setCheckFlag(boolean checkFlag) {
        this.checkFlag = checkFlag;
    }
}
